/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

/** Repraesentiert einen Schild, den ein Held traegt. */
public class SchildWert {

    /** Bruchfaktor */
    private int bf;

    /** Initiativemodifikator */
    private int ini;

    /** Der Name des Schildes. */
    private String name;

    /** Der Typ Schild um den es sich hier handelt. */
    private Schild schild;

    /** Die Groesse der Deckung gegen Fernkampfangriffe. */
    private SchildTyp typ;

    /** Waffenmodifikator Attacke */
    private int wmAt;

    /** Waffenmodifikator Parade */
    private int wmPa;

    public SchildWert( final Schild schild ) {

        setSchild( schild );
    }

    public int getBf() {
        return bf;
    }

    public void setBf( final int bf ) {
        this.bf = bf;
    }

    public int getIni() {
        return ini;
    }

    public void setIni( final int ini ) {
        this.ini = ini;
    }

    public String getName() {
        return name;
    }

    public void setName( final String name ) {
        this.name = name;
    }

    public SchildTyp getTyp() {
        return typ;
    }

    public void setTyp( final SchildTyp typ ) {
        this.typ = typ;
    }

    public Schild getSchild() {
        return schild;
    }

    public void setSchild( final Schild schild ) {
        this.schild = schild;

        setBf( schild.getBf() );

        setIni( schild.getIni() );

        setName( schild.getName() );

        setTyp( schild.getTyp() );

        setWmAt( schild.getWmAt() );

        setWmPa( schild.getWmPa() );
    }

    public int getWmAt() {
        return wmAt;
    }

    public void setWmAt( final int wmAt ) {
        this.wmAt = wmAt;
    }

    public int getWmPa() {
        return wmPa;
    }

    public void setWmPa( final int wmPa ) {
        this.wmPa = wmPa;
    }

    public int getParade( final Held held ) {

        int pa = held.getStufe( Basiswert.PAB );

        final Sonderfertigkeit[] schildkampf = new Sonderfertigkeit[]{
                Sonderfertigkeit.SCHILDKAMPF_I,
                Sonderfertigkeit.SCHILDKAMPF_II
        };

        for ( final Sonderfertigkeit sonderfertigkeit : schildkampf ) {

            if ( held.hat( sonderfertigkeit ) ) {

                pa += 3;
            }
        }

        pa += wmPa;

        return pa;
    }
}
