/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;

/**
 * Die Implementierung fuer Sonderfertigkeiten die keine Varianten besitzen.
 *
 * @author Bernhard Jung
 */
class VariantenKeine implements Varianten {

    /**
     * @throws UnsupportedOperationException
     */
    public void einfuegen( final String variante ) {
        throw new UnsupportedOperationException(
                "Die Sonderfertigkeit unterstuetzt keine Varianten");
    }

    /**
     * @return eine leere Liste.
     */
    @SuppressWarnings( "unchecked" )
    public List< Object > getVarianten(final Held held) {
        return Collections.EMPTY_LIST;
    }

    /** @return false */
    public boolean hatVarianten() {
        return false;
    }

    /** @return false */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "keine Varianten";
    }
}
