/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Liefert fuer einen Helden eine Liste mit allen Gelaendekunden die er noch
 * nicht besitzt.
 *
 * @author Bernhard Jung
 */
class SonderfertigkeitBonusGelaendekunde implements
        SonderfertigkeitBonus {

    /** Eine Beschreibung der Liste. */
    private static final String BESCHREIBUNG =
        I18n.i18n( "SonderfertigkeitBonusGelaendekunde" );

    public final boolean enthaelt( final Sonderfertigkeit sonderfertigkeit ) {
        return sonderfertigkeit.istGelaendekunde();
    }

    public final double getGpKosten() {
        return Sonderfertigkeit.DSCHUNGELKUNDIG.getApKosten() / 100;
    }

    /** @return alle Gelaendekunden die der Held noch nicht besitzt. */
    public final List< SonderfertigkeitWert > getWerte( final Held held ) {

        final List< SonderfertigkeitWert > gelaendekunden;
        gelaendekunden = new ArrayList< SonderfertigkeitWert >();

        for ( final Sonderfertigkeit gelaendekunde :
            Sonderfertigkeit.getGelaendekunden() ) {

            if ( ! held.hat( gelaendekunde ) ) {

                final SonderfertigkeitWert wert;
                wert = new SonderfertigkeitWert( gelaendekunde );

                gelaendekunden.add( wert );
            }

        }

        return gelaendekunden;
    }

    /** @return etwas wie 'eine Gelaendekunde nach Wahl' */
    @Override
    public final String toString() {
        return BESCHREIBUNG;
    }

    public final IXMLElement toXml() {

        return new XMLElement( HeldenbausteinXmlTags.TAG_GELAENDEKUNDE );
    }
}
