/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.moeglicheprof;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Laesst alle Professionen, mit Ausnahme einer vorgegebenen Liste, zu.
 *
 * @author Bernhard Jung
 *
 */
public class MoeglicheProfessionenAlleAusser implements MoeglicheProfessionen {

    private static final String ALLE_AUSSER =
        I18n.i18n( "MoeglicheProfessionenAlleAusser" );

    /** Die Professionen die nicht moeglich sind. */
    private final Collection< Profession > professionen;

    private final String toString;

    /**
     * @param dieProfessionen die nicht moeglich sind.
     */
    public MoeglicheProfessionenAlleAusser(
            final Collection< Profession > dieProfessionen ) {

        professionen = new HashSet< Profession >( dieProfessionen );

        final List< Profession > liste =
            new ArrayList< Profession >( professionen );

        Collections.sort( liste );

        toString = ALLE_AUSSER + " "
                 + Strings.erzeugeAufzaehlungMitUnd( liste );
    }

    /** {@inheritDoc} */
    public Collection< Profession > getProfessionen() {

        final List< Profession > liste =
            new ArrayList< Profession >( professionen );

        Collections.sort( liste );

        return liste;
    }

    /** {@inheritDoc} */
    public boolean istMoeglich(final Profession profession) {
        return ! professionen.contains( profession );
    }

    /** @return eine Beschreibung welche Professionen moeglich sind. */
    @Override
    public String toString() {
        return toString;
    }

    /** {@inheritDoc} */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_PROFESSIONEN );

        final IXMLElement alleAusserElement = new XMLElement( TAG_ALLE_AUSSER );

        for ( final Profession profession : professionen ) {

            final IXMLElement profElement = new XMLElement( TAG_PROFESSION );

            profElement.setContent( profession.getId() );

            alleAusserElement.addChild( profElement );
        }

        element.addChild( alleAusserElement );

        return element;
    }

}
