/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factorys;

import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.*;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;

import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.SchildTyp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class FactorySchilde extends Factory< Schild > {

    @Override
    public final List< Schild > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< Schild > schilde = new ArrayList< Schild >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement schildElement : wurzelElement.getChildren() ) {

            final String id = schildElement.getAttribute( ATTR_ID );

            final Schild schild = speicher.getSchild( id );

            schild.setBf( getInt( schildElement, ATTR_BF ) );

            schild.setIni( getInt( schildElement, ATTR_INI ) );

            if ( schildElement.hasAttribute( ATTR_NAME ) ) {

                schild.setName( schildElement.getAttribute( ATTR_NAME ) );
            }

            final String typId = schildElement.getAttribute( ATTR_TYP );

            schild.setTyp( SchildTyp.getTyp( typId ) );

            schild.setWmAt( getInt( schildElement, ATTR_WM_AT ) );

            schild.setWmPa( getInt( schildElement, ATTR_WM_PA ) );

            schilde.add( schild );
        }

        return schilde;
    }
}
