/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_KOSTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_RASSE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOSTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOSTEN_RASSE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_NORMAL;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_RASSE;
import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.kosten.Kosten;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenRasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

/**
 * Erzeugt die Kosten fuer eine Rasse, Kultur oder Profession.
 *
 * @author Bernhard Jung
 */
public class FactoryKostenIn {

    /**
     * @param wurzelElement des XML-Dokuments.
     * @return ein Objekt, das die Kosten eines Heldenbausteins bestimmt.
     */
    public final Kosten erzeugen( final IXMLElement wurzelElement ) {

        final IXMLElement kostenElement =
            wurzelElement.getFirstChildNamed( TAG_KOSTEN );

        final IXMLElement firstChild =
            kostenElement.getChildren().get( 0 );

        if ( firstChild.getName().equals( TAG_KOSTEN_RASSE ) ) {
            // Das ist der Fall dass eine bestimmte Rasse andere Kosten zahlen
            // muss.

            // Erst mal die Rasse ermitteln und die Kosten die sie zahlen muss.
            final IXMLElement rasseElement;
            rasseElement = firstChild.getFirstChildNamed( TAG_RASSE );

            final String rasseId;
            rasseId = rasseElement.getAttribute( ATTR_RASSE );

            final Rasse rasse = Datenspeicher.getInstanz().getRasse( rasseId );

            final String kostenRasseString =
                rasseElement.getAttribute( ATTR_KOSTEN );

            final int kostenRasse = Integer.valueOf( kostenRasseString );

            // Jetzt die Kosten die jede andere Rasse zahlen muss.
            final String kostenNormalString =
                firstChild.getFirstChildNamed( TAG_NORMAL ).getContent();

            final int kostenNormal = Integer.valueOf( kostenNormalString );

            return new KostenRasse( kostenNormal, rasse, kostenRasse );

        } else {
            // Der Heldenbaustein hat fuer jeden die selben Kosten, die Kosten
            // sind fest.
            final String kostenString = firstChild.getContent();

            final int kosten = Integer.valueOf( kostenString );

            return new KostenFest( kosten );
        }
    }
}
