/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_LEITSF;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitWertBonus;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Erzeugt die Leitsonderfertigkeiten einer Kultur aus dem XML-Dokument.
 */
public class FactorySonderfertigkeitWertIn extends FactorySonderfertigkeitIn {

    /**
     * Erzeugt die Leitsonderfertigkeiten einer Kultur aus dem XML-Dokument.
     *
     * @param wurzelElement des XML-Dokuments
     * @return die Leitsonderfertigkeiten der Kultur.
     */
    public final List< SonderfertigkeitWertBonus > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< SonderfertigkeitWertBonus > werte =
            new ArrayList< SonderfertigkeitWertBonus >();

        final IXMLElement leitsfElement =
            wurzelElement.getFirstChildNamed( TAG_LEITSF );

        if ( null != leitsfElement ) {

            for ( final IXMLElement wertElement : leitsfElement.getChildren() ) {

                final Sonderfertigkeit sonderfertigkeit =
                    ermitteleSonderfertigkeit( wertElement );

                final String variantenId = wertElement.getAttribute( ATTR_VARIANTE );

                final SonderfertigkeitWert wert;

                if ( ( null != variantenId ) ) {

                    final String variante;

                    if (  I18n.hatLokalisierungstext( variantenId )) {

                        variante = I18n.i18n( variantenId );

                    } else {

                        variante = variantenId;
                    }

                    wert = new SonderfertigkeitWert( sonderfertigkeit,
                                                         variante );

                } else {

                    wert = new SonderfertigkeitWert( sonderfertigkeit );
                }

                werte.add( new SonderfertigkeitWertBonus(
                        wert, variantenId ) );

            }

        }

        return werte;
    }
}
