/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BESONDERER_BESITZ;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.BesondererBesitz;

public class FactoryStringsIn {

    /**
     * Erzeugt die Moeglichkeiten fuer den Vorteil Besonderer Besitz.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Moeglichkeiten fuer Besonderer Besitzt. Eine leere Liste,
     *         wenn die Profession keinen besonderen Besitzt erlaubt.
     */
    public final List< BesondererBesitz > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< BesondererBesitz > besitze =
            new ArrayList< BesondererBesitz >();

        final IXMLElement stringsElement =
            wurzelElement.getFirstChildNamed( TAG_BESONDERER_BESITZ );

        if ( null != stringsElement ) {

            for ( final IXMLElement besitzElement :
                stringsElement.getChildren() ) {

                final String besitz = besitzElement.getContent();

                besitze.add( new BesondererBesitz( besitz ) );
            }
        }

        return besitze;
    }
}
