/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BIS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_ERGEBNIS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VON;
import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.util.Wuerfeltabelle;

/**
 * Erzeugt aus dem entsprechenden XML-Element eine Wuerfeltabelle, zB fuer die
 * Haar- oder Augenfarbe eines Helden.
 *
 * @author Bernhard Jung
 *
 */
public class FactoryWuerfeltabelleIn {

    /**
     * @param tabellenElement Das XML-Element das die Eintraege fuer die
     *      Wuerfeltabelle enthaelt.
     * @return die aus {@code tabellenElement} ermittelte Wuerfeltabelle.
     */
    public final Wuerfeltabelle erzeugen( final IXMLElement tabellenElement ) {

        final Wuerfeltabelle tabelle = new Wuerfeltabelle();

        for ( final IXMLElement eintrag : tabellenElement.getChildren() ) {

            // Den Wert ermitteln ab dem ein Ergebnis erwuerfelt wird.
            final String vonString = eintrag.getAttribute( ATTR_VON );

            final int von = Integer.valueOf( vonString );

            // Den Wert ermitteln bis zu dem ein Ergebnis erwuerfelt wird.
            final String bisString = eintrag.getAttribute( ATTR_BIS );

            final int bis = Integer.valueOf( bisString );

            // Das Ergebnis das erwuerfelt wird ermitteln.
            final String ergebnis = eintrag.getAttribute( ATTR_ERGEBNIS );

            tabelle.eintragen( von, bis, ergebnis );
        }

        return tabelle;
    }
}
