/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;

/**
 * Fasst die andern Panel zur Auswahl der Eigenschaften, Talente usw. zusammen.
 *
 * @author Bernhard Jung
 *
 */
public class PanelSteigernErschaffung extends JPanel {

    /** Zeigt die Basiswerte an. */
    private final PanelBasiswerte basiswerte;

    /** Erlaubt es die Eigenschaften zu waehlen. */
    private final PanelEigenschaften eigenschaften;

    /** Der Held, der aktuell gesteigert wird. */
    private Held held;

    /** Die allgemeinen Informationen wie Alter und Haarfarbe. */
    private final PanelHeldendaten heldendaten;

    /** Zeigt alle Regelkonflikte bei der Erschaffung an. */
    private final PanelProbleme probleme;

    /** Erlaubt es die Sonderfertigkeiten zu waehlen. */
    private final PanelSonderfertigkeiten sonderfertigkeiten;

    /** Erlaubt es die Talente zu waehlen. */
    private final PanelTalente talente;

    /** Erlaubt es die Vor-/Nachteile zu waehlen. */
    private final PanelVorNachteile vorNachteile;

    /** Erlaubt es die Zauber zu waehlen. */
    private final PanelZauber zauber;

    PanelSteigernErschaffung() {
        super( new BorderLayout() );

        basiswerte = new PanelBasiswerte();

        eigenschaften = new PanelEigenschaften();

        heldendaten = new PanelHeldendaten();

        probleme = new PanelProbleme();

        sonderfertigkeiten = new PanelSonderfertigkeiten( false );

        talente = new PanelTalente( false );

        vorNachteile = new PanelVorNachteile();

        zauber = new PanelZauber( false );

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab(
                i18n( "PanelSteigern.Heldendaten" ),
                erzeugeFrame(
                        i18n( "PanelSteigern.Heldendaten" ),
                        heldendaten ) );

        tabbedPane.addTab(
                i18n( "PanelSteigern.Eigenschaften" ) + "/" +
                i18n( "PanelSteigern.Basiswerte" ),
                erzeugeEigenschaftenBasiswertPanel() );

        tabbedPane.addTab(
                i18n( "PanelSteigern.Talente" ),
                talente );

        tabbedPane.addTab(
                i18n( "PanelSteigern.Sonderfertigkeiten" ),
                sonderfertigkeiten );

        tabbedPane.addTab(
                i18n( "PanelSteigern.VorNachteile" ),
                vorNachteile );

        tabbedPane.addTab(
                i18n( "PanelSteigern.Zauber" ),
                zauber );

        tabbedPane.addTab(
                i18n( "PanelSteigern.Probleme" ),
                erzeugeFrame( i18n( "PanelSteigern.Probleme" ), probleme ) );

        add( tabbedPane );
    }

    private JPanel erzeugeEigenschaftenBasiswertPanel() {

        final FormLayout layout = new FormLayout(
                "D:G, 4dlu, D:G",   // spalten
                "F:P" ); // zeilen

        final JPanel panel = new JPanel( layout );
        final CellConstraints cc = new CellConstraints();

        panel.add( erzeugeFrame(
                i18n( "PanelSteigern.Eigenschaften" ), eigenschaften ),
                cc.xy( 1, 1 ) );

        panel.add( erzeugeFrame(
                i18n( "PanelSteigern.Basiswerte" ), basiswerte ),
                cc.xy( 3, 1 )  );

        return panel;
    }

    /**
     * Erzeugt einen {@link SimpleInternalFrame}.
     *
     * @param titel fuer den Frame.
     * @param panel Der Inhalt des Frames.
     * @return einen Frame.
     */
    private final SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JPanel panel ) {

        final SimpleInternalFrame frame;
        frame = new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( panel );

        return frame;
    }

    /**
     * @return der Held der aktuell gesteigert wird.
     */
    public final Held getHeld() {
        return held;
    }

    /**
     * Bestimmt den Manager und damit den Held, der gerade erschaffen wird.
     *
     * @param manager fuer die Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        held = manager.getHeld();

        heldendaten.setHeld( held );

        eigenschaften.setHeld( held );

        basiswerte.setHeld( held );

        probleme.setManager( manager );

        sonderfertigkeiten.setManager( manager );

        talente.setHeld( held );

        vorNachteile.setManager( manager );

        zauber.setHeld( held );
    }

}
