/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.DialogEinstellungen;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class ActionEinstellungen extends AbstractAction {

    /** Das Tastenkurzel mit dem die Aktion ausgeloest werden kann. */
    private static final String TASTEN = 
        I18n.i18n( "ActionEinstellungen.Tasten" );

    /**
     * Die Taste mit der die Aktion im Menue direkt angesprungen werden kann.
     */
    private static final String MNEMONIC = 
        I18n.i18n( "ActionEinstellungen.Mnemonic" );

    /** Das Hauptfenster der Anwendung. */
    private final JFrame hauptfenster;

    /** Der Dialog den die Action anzeigt. */
    private DialogEinstellungen dialogEinstellungen;

    public ActionEinstellungen( final JFrame dasHauptfenster ) {

        super( I18n.i18n( "ActionEinstellungen.Name" ),
                Util.ladeIcon( "sprache" ) );

        hauptfenster = dasHauptfenster;

        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }

    /**
     * Macht den Dialog fuer die Einstellungen sichtbar oder unsichtbar.
     */
    public void actionPerformed(ActionEvent e) {
        
        if ( null == dialogEinstellungen ) {
            
            dialogEinstellungen = new DialogEinstellungen( hauptfenster );
            
            dialogEinstellungen.setLocationRelativeTo( hauptfenster );
        }
        
        dialogEinstellungen.setVisible( ! dialogEinstellungen.isVisible() );
    }

}
