/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.daten.Datenkonstanten;
import de.bernhardjung.dsaprogramm.daten.ProfessionenLader;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaProfession;

public class ActionLadenProfession extends ActionLadenHeldenbaustein {

    private final PanelMetaProfession panelProfession;

    private ProfessionenLader professionenLader;

    public ActionLadenProfession( final PanelMetaProfession dasPanelProfession ) {
        super( "ActionLadenProfession.Name",
                "ActionLadenProfession.Mnemonic",
                Datenkonstanten.DATEIERWEITERUNG_PROFESSION,
                Datenkonstanten.VERZEICHNIS_PROFESSIONEN,
                dasPanelProfession );

        panelProfession = dasPanelProfession;

        try {
            professionenLader = new ProfessionenLader();
        } catch ( final Exception e ) {
            // ignorieren
        }
    }

    @Override
    public void laden( final File datei ) throws Exception {

        try {

            final InputStream in = new FileInputStream( datei );

            final Profession profession =
                professionenLader.ladeProfession( datei.getName(), in );

            panelProfession.anzeigen( profession );

        } catch ( final FileNotFoundException ex ) {

            // ignorieren. Das die Datei existiert haben wir ja gerade
            // getestet.

        }
    }

}
