/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.AbstractListModel;

public class AenderbaresListModel< T > extends AbstractListModel {

    private final List< T > elemente = new ArrayList< T >();

    public void add( final T element ) {
        elemente.add( element );

        fireIntervalAdded( this, elemente.size(), elemente.size() );
    }

    /**
     * Loescht alle Elemente im Model.
     */
    public void clear() {

        final int size = getSize();

        if ( size > 0 ) {

            elemente.clear();

            fireIntervalRemoved( this, 0, size - 1 );
        }
    }

    /**
     * Prueft ob ein Element in der Liste vorhanden ist.
     *
     * @param element
     *            nach dem gesucht werden soll.
     * @return {@code true}, wenn das Element im Model gespeichert ist, sonst
     *         {@code false}.
     */
    public boolean contains( final T element ) {
        return elemente.contains( element );
    }

    /**
     * @return alle Elemente aus dem Model.
     */
    public List< T > getElemente() {
        return new ArrayList< T >( elemente );
    }

    /**
     * @param index
     *            des gesuchten Elements.
     * @return das Element an Position {@code index} im Model.
     */
    public T getElementAt( final int index ) {
        return elemente.get( index );
    }

    /**
     * @return die Anzahl Elemente im Model.
     */
    public int getSize() {
        return elemente.size();
    }

    /**
     * Entfernt ein Element aus dem Model.
     * @param index
     *            des Elements, das entfernt werden soll.
     * @return das entfernte Element.
     */
    public T remove( final int index ) {

        final T element = elemente.remove( index );

        fireIntervalRemoved( this, index, index );

        return element;
    }

    /**
     * Aendert die im Model gespeicherten Elemente.
     *
     * @param neueElemente fuer das Model.
     */
    public void setElemente( final Collection< T > neueElemente ) {
        elemente.clear();

        elemente.addAll( neueElemente );

        fireContentsChanged( this, 0, elemente.size() - 1 );
    }
}
