/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class PanelMetaFernkampfwaffen extends JPanel {

    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaFernkampfwaffen.Entfernen" );

    private static final String HINWEIS_DATENQUELLE =
        I18n.i18n( "PanelMetaFernkampfwaffen.HinweisDatenquelle" );

    private static final String ID =
        I18n.i18n( "PanelMetaFernkampfwaffen.Id" );

    private static final String NAME =
        I18n.i18n( "PanelMetaFernkampfwaffen.Name" );

    private static final String REICHWEITEN =
        I18n.i18n( "PanelMetaFernkampfwaffen.Reichweiten" );

    private static final String TALENT =
        I18n.i18n( "PanelMetaFernkampfwaffen.Talent" );

    private static final String TP =
        I18n.i18n( "PanelMetaFernkampfwaffen.Tp" );

    private static final String TP_PLUS =
        I18n.i18n( "PanelMetaFernkampfwaffen.TpPlus" );

    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaFernkampfwaffen.Uebernehmen");

    /** Die Liste der Fernwaffen. */
    private final JList fernwaffen;

    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Der Name. */
    private final JTextField name;

    /** Die Spinner zu Einstellen der Reichweiten. */
    private final List< JSpinner > reichweiten;

    /** Da sind die Ruestungen gespeichert. */
    private final Datenspeicher speicher;

    /** Das Talent mit dem die Waffe gefuehrt wird. */
    private final JComboBox talent;

    /** Das Textfeld zum eingeben der Trefferpunkte. */
    private final JTextField tp;

    /** Die Spinner zum Einstellen der Trefferpunkte aus Reichweite. */
    private final List< JSpinner > tpPlus;

    /**
     * Der Knopf mit dem die eingegebene Ruestung in die Liste der Ruestungen
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    public PanelMetaFernkampfwaffen() {
        speicher = Datenspeicher.getInstanz();

        final Vector< Fernkampfwaffe > vector =
            new Vector< Fernkampfwaffe >( speicher.getFernkampfwaffen() );

        fernwaffen = new JList( vector );
        fernwaffen.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        fernwaffen.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });

        hinweisGezeigt = false;

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        name = new JTextField();

        reichweiten = new ArrayList< JSpinner >( 5 );
        for ( int i = 0; i < 5 ; ++i ) {
            reichweiten.add( Util.erzeugeSpinner( 0, 0, 999, 5 ) );
        }

        final Vector< Talent > talente = new Vector< Talent >();
        for ( final Talent talent : Talentgruppe.KAMPF ) {

            if ( ((Kampftechnik) talent).istFernkampfTechnik() ) {

                talente.add( talent );
            }
        }
        talent = new JComboBox( talente );

        tp = new JTextField();
        tp.setBackground( Util.FEHLERFARBE );
        tp.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        tpPlus = new ArrayList< JSpinner >( 5 );
        for ( int i = 0; i < 5 ; ++i ) {
            tpPlus.add( Util.erzeugeSpinner( 0, -9, 9, 1 ) );
        }

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final int index = fernwaffen.getSelectedIndex();

        if ( 0 <= index ) {

            final Fernkampfwaffe waffe = (Fernkampfwaffe) fernwaffen.getSelectedValue();

            id.setText( waffe.getId().replace( "Waffe.", "" ) );

            name.setText( waffe.getName() );

            final List< Integer > reichweitenWerte = waffe.getReichweiten();
            final List< Integer > tpPlusWerte = waffe.getTpPlus();

            for ( int i = 0 ; i < 5 ; ++i ) {

                reichweiten.get( i ).setValue( reichweitenWerte.get( i ) );

                tpPlus.get( i ).setValue( tpPlusWerte.get( i ) );
            }

            talent.setSelectedItem( waffe.getTalent() );

            tp.setText( waffe.getTp().toString() );

        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        final String spalten = "7dlu, 200px:G, 4dlu, P, 4dlu, P, P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
            "4dlu, P, 4dlu, P, P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( fernwaffen ), cc.xywh( 2, 2, 1, 14 ) );

        add( entfernen, cc.xy( 2, 17 ) );

        int y = 0;

        add( new JLabel( NAME ), cc.xy ( 4, y+=2 ) );
        add( name,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( ID ), cc.xy ( 4, y+=2 ) );
        add( id,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( TP ), cc.xy ( 4, y+=2 ) );
        add( tp,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( TALENT ), cc.xy( 4, y+=2 ) );
        add( talent,               cc.xy( 6, y ) );

        add( new JLabel( REICHWEITEN ),   cc.xy( 4, y+=2 ) );
        add( erzeugePanel( reichweiten ), cc.xy( 6, y ) );

        add( new JLabel( TP_PLUS ),  cc.xy( 4, y+=2 ) );
        add( erzeugePanel( tpPlus ), cc.xy( 6, y ) );

        add( buttonpanel, cc.xyw( 4, y+=2, 4 ) );
    }

    /**
     * Entfernt die in {@link #fernwaffen} geweahlte Waffe aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final int index = fernwaffen.getSelectedIndex();

        if ( 0 <= index ) {

            final Fernkampfwaffe fernwaffe =
                (Fernkampfwaffe) fernwaffen.getSelectedValue();

            speicher.entfernen( fernwaffe );

            final List< Fernkampfwaffe > liste = speicher.getFernkampfwaffen();

            final Vector< Fernkampfwaffe > vector =
                new Vector< Fernkampfwaffe >( liste );

            fernwaffen.setListData( vector );

            speichern();
        }
    }

    /**
     * Ordnet die Spinner auf einem Panel in einer horziontalen Linie an.
     */
    private JPanel erzeugePanel( final List< JSpinner > spinner ) {

        final FormLayout layout =
            new FormLayout( "P, 2dlu, P, 2dlu, P, 2dlu, P, 2dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        for( final JSpinner s : spinner ) {

            builder.append( s );
        }

        return builder.getPanel();
    }

    /**
     * Liefert die in den Spinnern eingestellten Zahlen.
     *
     * @param spinner
     *            deren Werte gesucht sind.
     * @return die Werte aus den Spinnern.
     */
    private List< Integer > getWerte( final List< JSpinner > spinner ) {

        final List< Integer > werte = new ArrayList< Integer >( spinner.size());

        for ( final JSpinner s : spinner ) {

            werte.add( (Integer) s.getValue() );
        }

        return werte;
    }

    /**
     * Prueft die Eingaben im Panel auf Korrektheit.
     *
     * @return {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            ergebnis = false;

        } else {

            id.setBackground( name.getBackground() );
        }

        if ( Tp.istTpFormel( tp.getText().trim() ) ) {

            tp.setBackground( name.getBackground() );

        } else {

            tp.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }

    /**
     * Speichert die Waffen auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereFernkampfwaffen();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( this, HINWEIS_DATENQUELLE );

            hinweisGezeigt = true;
        }

    }

    /**
     * Uebernimmt die im Panel eingegebenen Fernwaffe in die Liste der
     * Fernwaffen.
     */
    private final void uebernehmen() {

        if ( ! pruefen() ) {
            return;
        }

        String idString = id.getText().trim();

        idString = "Waffe." + idString;

        final Fernkampfwaffe waffe = speicher.getFernkampfwaffe( idString );

        waffe.setName( name.getText().trim() );

        waffe.setReichweiten( getWerte( reichweiten ) );

        waffe.setTalent( (Kampftechnik) talent.getSelectedItem() );

        waffe.setTp( new Tp( tp.getText().trim() ) );

        waffe.setTpPlus( getWerte( tpPlus ) );

        speicher.einfuegen( waffe );

        speichern();

        final List< Fernkampfwaffe > liste = speicher.getFernkampfwaffen();

        final Vector< Fernkampfwaffe > vector = new Vector< Fernkampfwaffe >( liste );

        fernwaffen.setListData( vector );
    }
}
