/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungGeschlecht;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Das Panel zum Angeben von Name, Muttersprache und zusaetzlichen Leitzaubern.
 *
 * @author Bernhard Jung
 */
public class PanelMetaKulturDiverses extends JPanel {

    /**
     * Der Text fuer den Knopf zum Berechnen der GP-Kosten.
     */
    private static final String BERECHNEN =
        I18n.i18n( "PanelMetaKulturDiverses.Berechnen" );

    /** Der Text neben dem Textfeld, zum Eingeben einer Id. */
    private static final String ID =
        I18n.i18n( "PanelMetaKulturDiverses.Id" );

    /** Der Text neben dem Spinner fuer die Kosten. */
    private static final String KOSTEN =
        I18n.i18n( "PanelMetaKulturDiverses.Kosten" );

    /**
     * Der Text neben dem Spinner zum einstellen der zusaetzlichen Leitzauber.
     */
    private static final String LEITZAUBER =
        I18n.i18n( "PanelMetaKulturDiverses.Leitzauber" );

    /** Der Text neben der Combobox zum waehlen der Muttersprache. */
    private static final String MUTTERSPRACHE =
        I18n.i18n( "PanelMetaKulturDiverses.Muttersprache" );

    /** Der Text neben dem Textfeld zum Eingeben des Namens. */
    private static final String NAME =
        I18n.i18n( "PanelMetaKulturDiverses.Name" );

    /** 
     * Der Text der Checkbox mit der man eine NameFactory fuer die Kultur 
     * waehlen kann 
     */
    private static final String NAMEN =
        I18n.i18n( "PanelMetaKulturDiverses.Namen" );

    /**
     * Der Text auf der Checkbox, mit der man waehlen kann, dass der Held
     * maennlich oder weiblich sein soll.
     */
    private static final String VORAUSSETZUNG =
        I18n.i18n( "PanelMetaKulturDiverses.Voraussetzung" );

    private final JCheckBox geschlechtCheckBox;

    private final JComboBox geschlechtComboBox;

    private final JTextField id;

    private final JSpinner kosten;

    private final JSpinner leitzauber;

    private final JComboBox muttersprache;

    private final JTextField name;
    
    private final JCheckBox nameFactoryCheckBox;
    
    private final JComboBox nameFactoryComboBox;

    private final PanelMetaKultur panelKultur;

    public PanelMetaKulturDiverses( final PanelMetaKultur dasPanelKultur ) {

        geschlechtComboBox = new JComboBox( Geschlecht.values() );
        geschlechtComboBox.setEnabled( false );
        
        geschlechtCheckBox = new JCheckBox( VORAUSSETZUNG );
        geschlechtCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                
                boolean selected = geschlechtCheckBox.isSelected();
                
                geschlechtComboBox.setEnabled( selected );
            }
        } );
        
        id = new JTextField();

        kosten = Util.erzeugeSpinner( 0, -9, 99, 1 );

        leitzauber = Util.erzeugeSpinner( 0, 0, 99, 1 );

        final Vector< Talent > sprachen =
            new Vector< Talent >( Talentgruppe.SPRACHEN );

        muttersprache = new JComboBox( sprachen );

        name = new JTextField();

        Datenspeicher speicher = Datenspeicher.getInstanz();
        nameFactoryComboBox = 
            new JComboBox( speicher.getNameFactorys().toArray() );
        nameFactoryComboBox.setEnabled( false );
        
        nameFactoryCheckBox = new JCheckBox( NAMEN );
        nameFactoryCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                
                boolean selected = nameFactoryCheckBox.isSelected();
                
                nameFactoryComboBox.setEnabled( selected );
            }
        } );
        
        panelKultur = dasPanelKultur;

        final JLabel kostenLabel = new JLabel();

        final JButton berechnen = new JButton( BERECHNEN );
        berechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Kultur kultur = new Kultur( "Kultur" );

                panelKultur.uebertrageDaten( kultur );

                final double kosten = kultur.berechneGpKosten();

                kostenLabel.setText( String.format( "%1$.1f", kosten ) );
            }
        });

        final FormLayout layout = new FormLayout( "P, 4dlu, F:P:G" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        builder.append( NAME, name );

        builder.append( ID, id );

        final FormLayout layout1 = new FormLayout( "P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder1 = new DefaultFormBuilder( layout1 );

        builder1.append( KOSTEN, kosten );
        builder1.append( berechnen, kostenLabel );

        builder.append( builder1.getPanel(), 3 );

        final FormLayout layout2 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder2 = new DefaultFormBuilder( layout2 );

        builder2.append( MUTTERSPRACHE, muttersprache );

        builder.append( builder2.getPanel(), 3 );

        final FormLayout layout3 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder3 = new DefaultFormBuilder( layout3 );

        builder3.append( LEITZAUBER, leitzauber );

        builder.append( builder3.getPanel(), 3 );

        final FormLayout layout4 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder4 = new DefaultFormBuilder( layout4 );

        builder4.append( geschlechtCheckBox, geschlechtComboBox );

        builder.append( builder4.getPanel(), 3 );
        
        final FormLayout layout5 = new FormLayout( "P, 4dlu, P" );
        
        final DefaultFormBuilder builder5 = new DefaultFormBuilder( layout5 );
        
        builder5.append( nameFactoryCheckBox, nameFactoryComboBox );

        builder.append( builder5.getPanel(), 3 );
    }

    /**
     * Zeigt einen Kultur im Panel an.
     *
     * @param kultur
     *            die angezeigt werden soll.
     */
    void anzeigen( final Kultur kultur ) {

        kosten.setValue( kultur.getGpKosten( null ) );

        anzeigenId( kultur.getId() );

        leitzauber.setValue( kultur.getAnzahlLeitzauber() );

        muttersprache.setSelectedItem( kultur.getMuttersprache() );

        name.setText( kultur.getName() );

        if ( kultur.hatNameFactory() ) {
            
            nameFactoryCheckBox.setSelected( true );
        
            nameFactoryComboBox.setSelectedItem( kultur.getNameFactory() );
        
        } else {
            
            nameFactoryCheckBox.setSelected( false );
        }
        
        geschlechtCheckBox.setSelected( false );

        for ( final Voraussetzung voraussetzung : kultur.getVoraussetzungen() ) {

            if ( voraussetzung instanceof VoraussetzungGeschlecht ) {

                final Geschlecht geschlecht =
                    ( (VoraussetzungGeschlecht) voraussetzung ).getGeschlecht();

                geschlechtComboBox.setSelectedItem( geschlecht );

                geschlechtCheckBox.setSelected( true );

                break;
            }
        }
    }

    private void anzeigenId( final String idString ) {

        final int index = idString.indexOf( "." );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );
    }

    /** @return die fuer die Kultur gewaehlte Id. */
    String getKulturId() {
        return "Kultur." + id.getText().trim();
    }

    /**
     * @return den Namen den der Benutzer fuer die Kultur eingegeben hat.
     */
    String getKulturName() {
        return name.getText();
    }

    /**
     * Setzt die Eintraege im Panel wieder zurueck.
     */
    public void reset() {

        geschlechtCheckBox.setSelected( false );

        id.setText( "" );

        kosten.setValue( 0 );

        leitzauber.setValue( 0 );

        name.setText( "" );

    }

    /**
     * Uebertraegt die im Panel gemachten Einstellungen auf eine Kultur.
     *
     * @param kultur
     *            die die Einstellungen erhalten soll.
     */
    @SuppressWarnings( "unchecked" )
    void uebertrageDaten( final Kultur kultur ) {

        kultur.setKosten( new KostenFest( (Integer) kosten.getValue() ) );

        kultur.setAnzahlLeitzauber( (Integer) leitzauber.getValue() );

        kultur.setMuttersprache( (Sprache) muttersprache.getSelectedItem() );

        kultur.setName( name.getText() );

        if ( nameFactoryCheckBox.isSelected() ) {
            
            final Object factory = nameFactoryComboBox.getSelectedItem();
            
            kultur.setNameFactory( (NameFactory) factory );
        
        } else {
            
            kultur.setNameFactory( null );
        }
            
        if ( geschlechtCheckBox.isSelected() ) {

            final Geschlecht geschlecht =
                (Geschlecht) geschlechtComboBox.getSelectedItem();

            final Voraussetzung voraussetzung =
                new VoraussetzungGeschlecht( geschlecht );

            kultur.setVoraussetzungen( Arrays.asList( voraussetzung ) );

        } else {

            kultur.setVoraussetzungen( Collections.EMPTY_LIST );
        }
    }
}
