/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

public class PanelMetaListe< T > extends JPanel {

    private final List< JCheckBox > checkBoxen;

    final List< T > elemente;

    PanelMetaListe( final List< T > dieElemente ) {

        checkBoxen = new ArrayList< JCheckBox >();

        elemente = new ArrayList< T >( dieElemente );

        aufbauenBenutzeroberflaeche();
    }

    protected void aufbauenBenutzeroberflaeche() {

        removeAll();

        initCheckBoxen();

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( new FormLayout( "P" ), this );

        builder.setDefaultDialogBorder();

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            builder.append( checkBoxen.get( i ) );
        }
    }

    final List< T > getGewaehlteElemente() {

        final List< T > gewaehlte = new ArrayList< T >();

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            if ( checkBoxen.get( i ).isSelected() ) {

                gewaehlte.add( elemente.get( i ) );
            }
        }

        return gewaehlte;
    }

    private void initCheckBoxen() {

        for ( int i = checkBoxen.size(), n = elemente.size(); i < n ; ++i ) {

            checkBoxen.add( new JCheckBox() );
        }

        for ( int i = 0, n = elemente.size(); i < n ; ++i ) {

            checkBoxen.get( i ).setSelected( false );
            checkBoxen.get( i ).setText( elemente.get( i ).toString() );
        }

    }

    /** Waehlt alle Eintraege ab. */
    @SuppressWarnings( "unchecked" )
    public void reset() {
        setGewaehlteElemente( Collections.EMPTY_LIST );
    }

    final void setGewaehlteElemente( final Collection< T > gewaehlte ) {

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final JCheckBox checkBox = checkBoxen.get( i );

            final boolean gewaehlt = gewaehlte.contains( elemente.get( i ) );

            checkBox.setSelected( gewaehlt );
        }
    }
}
