package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.AbstractTableModel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Ein Editor der im Editor fuer die NameFactorys benutzt wird, um die 
 * Namenlisten zu waehlen, die zum Erzeugen eines Namens verwendet wird.
 * 
 * Der Editor erlaubt es zu jeder verwendeten Namenliste einen Schluessel zu
 * waehlen. Der Schluessel kann in der Formatvorlage fuer einen Namen verwendet
 * werden und wird dort durch einen Namen aus der entsprechenden Liste ersetzt.
 * 
 * Der Editor zeigt oben eine Tabelle die die Namenslisten mit den zugehoerigen
 * Schluesseln anzeigt und darunter befindet sich eine Combobox mit der man
 * einen neue Namensliste waehlen kann und ein Textfeld in das man den 
 * zugehoerigen Schluessel eintragen kann.
 */
class PanelMetaNameFactorysNamenlisten extends JPanel implements Observer {

    /**
     * Der Text auf dem Knopf mit dem man eine Schluessel-Namenliste-Kombination
     * aus der Tabelle entfernt.
     */
    private static final String ENTFERNEN = 
        I18n.i18n( "PanelMetaNameFactoriesNamenlisten.Entfernen" );
    
    /** 
     * Der Titel der Tabellenspalte in der die Namenlisten angezeigt werden.
     * Auch der Text vor der Combobox mit der man eine Namenliste waehlt. 
     */
    private static final String NAMENLISTE = 
        I18n.i18n( "PanelMetaNameFactoriesNamenlisten.Namenliste" );
    
    /** 
     * Der Titel der Tabellenspalte in der die Schluessel angezeigt werden.
     * Auch der Text vor dem Textfeld in das man einen Schluessel eingeben kann.
     */
    private static final String SCHLUESSEL = 
        I18n.i18n( "PanelMetaNameFactoriesNamenlisten.Schluessel" );

    /** 
     * Der Text auf dem Knopf mit dem man die gewaehlte Schluessel-Namenliste-
     * Kombi in die Tabelle einfuegt.. 
     */
    private static final String UEBERNEHMEN = 
        I18n.i18n( "PanelMetaNameFactoriesNamenlisten.Uebernehmen" );
    
    /**
     * Wenn das Textfeld fuer den Schluessel leer ist wird es rot dargestellt, 
     * damit der Beutzer, weiß dass er etwas eingeben muss. Ist etwas eingegeben
     * wird die Farbe wieder auf die normale Einstellunge gestellt. Die normale
     * Farbe wird hier gespeichert. 
     */
    private Color hintergrundfarbeTextField;
    
    /**
     * Die Combobox, um eine neue Namenliste zu waehlen.
     */
    private JComboBox namenlistenComboBox;
    
    /**
     * Die Tabelle die die gewaehlten Namenlisten anzeigt.
     */
    private JTable namenlistenTabelle;
    
    /**
     * Das TableModel fuer die Tabelle.
     */
    private NamenlistenTableModel namenlistenTableModel;
    
    /**
     * Das Textfeld in das der Schluessel fuer eine Namenliste eingegeben 
     * werden kann.
     */
    private JTextField schluesselTextField;

    /**
     * Der Knopf mit dem sich die gewaehlte Namenliste-Schluessel-Kombination
     * zur Liste der Namenlisten hinzufuegenl laesst.
     */
    private JButton uebernehmen;
    
    public PanelMetaNameFactorysNamenlisten() {
    
        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e ) {
               hinzufuegen();
           } 
        } );
        
        namenlistenComboBox = new JComboBox();
        Datenspeicher datenspeicher = Datenspeicher.getInstanz();
        datenspeicher.addObserver( this );
        fuelleComboBox();
        
        namenlistenTableModel = new NamenlistenTableModel();
        
        namenlistenTabelle = new JTable( namenlistenTableModel );
        
        schluesselTextField = new JTextField();
        schluesselTextField.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( DocumentEvent e ) {}
            public void insertUpdate( DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( DocumentEvent e ) {
                pruefen();
            }
        } );
        schluesselTextField.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegen();
            }
        } );
    
        hintergrundfarbeTextField = schluesselTextField.getBackground();
        
        aufbauenBenutzeroberflaeche();
        
        pruefen();
    }
    
    private void aufbauenBenutzeroberflaeche() {

        final JPanel hauptpanel = new JPanel( new BorderLayout()  );
        
        hauptpanel.add( erzeugeOberenBereich(), BorderLayout.CENTER );
        
        hauptpanel.add( erzeugeUnterenBereich(), BorderLayout.PAGE_END );
        
        final FormLayout layout = new FormLayout( "F:400px:G", "F:P:G" );
        
        final DefaultFormBuilder builder = 
            new DefaultFormBuilder( layout, this );
        builder.setDefaultDialogBorder();
        
        builder.append( hauptpanel );
    }
    
    /**
     * @return Der oberer Bereich mit der Tabelle. 
     */
    private JComponent erzeugeOberenBereich() {
        
        final JScrollPane scrollPane = 
            new JScrollPane( namenlistenTabelle );
        scrollPane.setPreferredSize( new Dimension( 300, 225 ) );
        
        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                
                final int index = namenlistenTabelle.getSelectedRow();
                
                if ( -1 != index ) {
                    
                    namenlistenTableModel.entfernen( index );
                }
            }
        } );
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * @return Der untere Bereich mit der Combobox und dem Textfeld.
     */
    private JComponent erzeugeUnterenBereich() {
        
        final FormLayout layout = new FormLayout( 
                "P, 4dlu, F:P:G",
                "7dlu, P, 4dlu, P, 4dlu" );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.nextLine( 1 );
        builder.append( NAMENLISTE, namenlistenComboBox );
        
        builder.nextLine( 2 );
        builder.append( SCHLUESSEL, schluesselTextField );
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        panel.add( builder.getPanel(), BorderLayout.PAGE_START );
        
        panel.add( uebernehmen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * Fuellt die Combobox erneut, wenn sich die Namenlisten geaendert haben.
     */
    private void fuelleComboBox() {
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        
        final List< Namenliste > namenlisten = speicher.getNamenlisten();
        
        namenlistenComboBox.setModel( 
                new DefaultComboBoxModel( namenlisten.toArray() ) );
    }
    
    /**
     * @return Die gewaehlten Schluessel-Namenlisten-Kombinationen.
     */
    public Map< String, Namenliste > getNamenlisten() {
        
        return namenlistenTableModel.getNamenlisten();
    }
    
    /**
     * Fuegt die aktuell gewaehlte Namenliste-Schluessel-Kombiniation in die
     * Tabelle ein.
     */
    private void hinzufuegen() {
        
        if ( ! pruefen() ) {
            
            return;
        }
        
        final String schluessel = schluesselTextField.getText().trim();
        
        final Namenliste namenliste = 
            (Namenliste) namenlistenComboBox.getSelectedItem();
        
        namenlistenTableModel.hinzufuegen( schluessel, namenliste );
        
        schluesselTextField.setText( "" );
    }
    
    /**
     * @return {@code true} wenn die eingegebenen Schluessel-Namenliste-Kombi
     *         korrekt ist und in die Tabelle eingefuegt werden kann, sonst
     *         {@code false}.
     */
    private boolean pruefen() {
        
        boolean eingabeOk = true;
        
        if ( schluesselTextField.getText().trim().equals( "" ) ) {
            
            schluesselTextField.setBackground( Util.FEHLERFARBE );
            
            eingabeOk = false;
        
        } else {
            
            schluesselTextField.setBackground( hintergrundfarbeTextField );
        }
        
        if ( -1 == namenlistenComboBox.getSelectedIndex() ) {
            
            eingabeOk = false;
        }
        
        uebernehmen.setEnabled( eingabeOk );
        
        return eingabeOk;
    }
    
    /**
     * Loescht die eingegebenen Daten aus dem Editor.
     */
    public void reset() {
        
        namenlistenTableModel.reset();
        
        schluesselTextField.setText( "" );
    }
    
    /**
     * @param listen
     *            Die Namenlisten, die im Editor angezeigt und bearbeitet werden
     *            sollen.
     */
    public void setNamenlisten( final Map< String, Namenliste > listen ) {
        
        namenlistenTableModel.setNamenlisten( listen );
    }
    
    /**
     * Wird aufgerufen, wenn sich etwas am Datenspeicher aendert.
     */
    public void update( Observable o, Object arg ) {
        
        if ( Datenspeicher.NAMENLISTEN.equals( arg ) ) {
            
            fuelleComboBox();
        }
    }
    
    /**
     * Das Model fuer die Tabelle mit den Namenlisten.
     */
    private static class NamenlistenTableModel extends AbstractTableModel {

        /** Der Index der Spalte mit den Schluesseln. */
        private static final int SPALTE_SCHLUESSEL = 0;
        /** Der Index der Spalte mit den Namenlisten. */
        private static final int SPALTE_NAMENLISTE = 1;
        
        private static final String[] SPALTEN_NAMEN = {
            SCHLUESSEL,
            NAMENLISTE
        };
        
        /**
         * Der Inhalt der Spalte mit den Namenlisten. Der Index in der Liste
         * entspricht dem Index einer Zeile in der Tabelle.
         */
        private final List< Namenliste > namenlistenListe;

        /**
         * Der Inhalt der Spalte mit den Schluesseln. Der Index in der Liste
         * entspricht dem Index einer Zeile in der Tabelle.
         */
        private final List< String > schluesselListe;
        
        public NamenlistenTableModel() {
        
            namenlistenListe = new ArrayList< Namenliste >();
            
            schluesselListe = new ArrayList< String >();
        }
        
        /**
         * Entfernt die gegebene Zeile aus der Tabelle.
         * 
         * @param zeile
         *            Die zu entfernende Zeile.
         */
        public void entfernen( int zeile ) {
            
            namenlistenListe.remove( zeile );
            
            schluesselListe.remove( zeile );
            
            fireTableRowsDeleted( zeile, zeile );
        }
        
        public int getColumnCount() {
            return SPALTEN_NAMEN.length;
        }

        @Override
        public String getColumnName( int column ) {
            
            return SPALTEN_NAMEN[ column ];
        }
    
        /**
         * @return Die in der Tabelle angezeigten Schluessel-Namenlisten-Kombis.
         */
        public Map< String, Namenliste > getNamenlisten() {
            
            final Map< String, Namenliste > namenlisten =
                new HashMap< String, Namenliste >();
            
            for ( int i = 0 ; i < namenlistenListe.size() ; ++i ) {
             
                namenlisten.put( 
                        schluesselListe.get( i ), 
                        namenlistenListe.get( i ) );
            }
            
            return namenlisten;
        }
        
        public int getRowCount() {
            
            return schluesselListe.size();
        }

        public Object getValueAt( int row, int column ) {
            
            switch ( column ) {
            case SPALTE_SCHLUESSEL:
                
                return schluesselListe.get( row );
                
            case SPALTE_NAMENLISTE:
                
                return namenlistenListe.get( row ).getBezeichnung();

            default:
                
                return "";
            }
        }
        
        /**
         * Fuegt der Tabelle eine neue Schluesse-Namenliste-Kombi hinzu.
         * Ein Schluessel kann nur einmal in der Tabelle vorkommen. 
         * Eine bestehende Kombination mit dem selben Schluessel wird 
         * ueberschrieben.
         */
        public void hinzufuegen( 
                final String schluessel,
                final Namenliste namenliste ) {
            
            int index = schluesselListe.indexOf( schluessel );
            
            if ( -1 == index ) {
            
                index = schluesselListe.size();
                
                schluesselListe.add( schluessel );
                
                namenlistenListe.add( namenliste );
            
                fireTableRowsInserted( index, index );
                
            } else {
                
                namenlistenListe.set( index, namenliste );
                
                fireTableRowsUpdated( index, index );
            }
        }
        
        /**
         * Loescht den Inhalt des Models.
         */
        public void reset() {
            
            namenlistenListe.clear();
            
            schluesselListe.clear();
            
            fireTableDataChanged();
        }
        
        /**
         * Fuellt die Tabelle mit den uebergebenene Namenlisten.
         */
        public void setNamenlisten( 
                final Map< String, Namenliste > neueListen ) {
            
            namenlistenListe.clear();
            schluesselListe.clear();
            
            for( final String s : neueListen.keySet() ) {
                
                schluesselListe.add( s );
                
                namenlistenListe.add( neueListen.get( s ) );
            }
            
            fireTableDataChanged();
        }
    }
}
