/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenProfession;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernProfession;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Das Panel zum Eingeben einer Kultur.
 *
 * @author Bernhard Jung
 *
 */
public class PanelMetaProfession extends JPanel {

    private static final String DIVERSES_FRAME =
        I18n.i18n( "PanelMetaProfession.Diverses" );

    private static final String DIVERSES_REITER =
        I18n.i18n( "PanelMetaProfession.Diverses.Reiter" );

    private static final String BESONDERER_BESITZ_FRAME =
        I18n.i18n( "PanelMetaProfession.BesondererBesitz" );

    private static final String BESONDERER_BESITZ_REITER =
        I18n.i18n( "PanelMetaProfession.BesondererBesitz.Reiter" );

    private static final String EMPF_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaProfession.EmpfohleneVorNachteile" );

    private static final String EMPF_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaProfession.EmpfohleneVorNachteile.Reiter" );

    private static final String LEITTALENTE_FRAME =
        I18n.i18n( "PanelMetaProfession.Leittalente" );

    private static final String NEU =
        I18n.i18n( "PanelMetaProfession.Neu" );

    private static final String LEITTALENTE_REITER =
        I18n.i18n( "PanelMetaProfession.Leittalente.Reiter" );

    private static final String MODIFIKATIONEN_FRAME =
        I18n.i18n( "PanelMetaProfession.Modifikationen" );

    private static final String MODIFIKATIONEN_REITER =
        I18n.i18n( "PanelMetaProfession.Modifikationen.Reiter" );

    private static final String NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaProfession.Nachteile" );

    private static final String NACHTEILE_REITER =
        I18n.i18n( "PanelMetaProfession.Nachteile.Reiter" );

    private static final String SONDERFERTIGKEITEN_FRAME =
        I18n.i18n( "PanelMetaProfession.Sonderfertigkeiten" );

    private static final String SONDERFERTIGKEITEN_REITER =
        I18n.i18n( "PanelMetaProfession.Sonderfertigkeiten.Reiter" );

    private static final String TALENTE_FRAME =
        I18n.i18n( "PanelMetaProfession.Talente" );

    private static final String TALENTE_REITER =
        I18n.i18n( "PanelMetaProfession.Talente.Reiter" );

    private static final String UNGE_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaProfession.UngeeigneteVorNachteile" );

    private static final String UNGE_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaProfession.UngeeigneteVorNachteile.Reiter" );

    private static final String VERBILLIGTE_SF_FRAME =
        I18n.i18n( "PanelMetaProfession.VerbilligteSonderfertigkeiten" );

    private static final String VERBILLIGTE_SF_REITER =
        I18n.i18n( "PanelMetaProfession.VerbilligteSonderfertigkeiten.Reiter" );

    private static final String VERBILLIGTE_VARIANTEN_FRAME =
        I18n.i18n( "PanelMetaProfession.VerbilligteVarianten" );

    private static final String VERBILLIGTE_VARIANTEN_REITER =
        I18n.i18n( "PanelMetaProfession.VerbilligteVarianten.Reiter" );

    private static final String VORAUSSETZUNGEN_FRAME =
        I18n.i18n( "PanelMetaProfession.Voraussetzungen" );

    private static final String VORAUSSETZUNGEN_REITER =
        I18n.i18n( "PanelMetaProfession.Voraussetzungen.Reiter" );

    private static final String VORTEILE_FRAME =
        I18n.i18n( "PanelMetaProfession.Vorteile" );

    private static final String VORTEILE_REITER =
        I18n.i18n( "PanelMetaProfession.Vorteile.Reiter" );

    private static final String ZAUBER_FRAME =
        I18n.i18n( "PanelMetaProfession.Zauber" );

    private static final String ZAUBER_REITER =
        I18n.i18n( "PanelMetaProfession.Zauber.Reiter" );

    private final PanelMetaBesondererBesitz besondererBesitz;

    private final PanelMetaProfessionDiverses diverses;

    private final PanelMetaListe< VorNachteil > empfohlene;

    private final PanelMetaLeittalente leittalente;

    private final PanelMetaModifikationen modifikationen;

    private final PanelMetaVorNachteile nachteile;

    private final PanelMetaSonderfertigkeiten sonderfertigkeiten;

    private final PanelMetaTalentboni talentboni;

    private final PanelMetaUngeeigneteVorNachteile ungeeignete;

    private final PanelMetaVerbilligteSonderfertigkeiten verbilligteSf;

    private final PanelMetaVerbilligteVarianten verbilligteVarianten;

    private final PanelMetaVoraussetzungen voraussetzungen;

    private final PanelMetaVorNachteile vorteile;

    private final PanelMetaZauberboni zauberboni;

    @SuppressWarnings( "unchecked")
    public PanelMetaProfession(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        besondererBesitz = new PanelMetaBesondererBesitz();

        diverses = new PanelMetaProfessionDiverses( this );

        empfohlene = new PanelMetaListe< VorNachteil>(
                VorNachteil.getVorNachteile() );

        leittalente = new PanelMetaLeittalente();

        modifikationen = new PanelMetaModifikationen(
                Collections.EMPTY_LIST );

        nachteile = new PanelMetaVorNachteile( VorNachteil.getNachteile(),
                                               NACHTEILE_REITER );

        sonderfertigkeiten = new PanelMetaSonderfertigkeiten();

        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.PROFESSION );

        ungeeignete = new PanelMetaUngeeigneteVorNachteile();

        verbilligteSf = new PanelMetaVerbilligteSonderfertigkeiten();

        verbilligteVarianten = new PanelMetaVerbilligteVarianten();

        voraussetzungen = new PanelMetaVoraussetzungen();

        vorteile = new PanelMetaVorNachteile( VorNachteil.getVorteile(),
                                              VORTEILE_REITER );

        zauberboni = new PanelMetaZauberboni();

        // Button Bar
        // **********
        final JButton neu = new JButton( NEU );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                neu();
            }
        });

        final JPanel buttonBar = new JPanel(
                new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar(
                    new JButton( new ActionLadenProfession( this ) ),
                    new JButton( new ActionSpeichernProfession(
                            hauptfenster, hauptpanel ) ),
                    neu ),
                    cc.xy( 2, 2 ) );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab( DIVERSES_REITER,
                           erzeugeFrame( DIVERSES_FRAME, diverses ) );

        tabbedPane.addTab( MODIFIKATIONEN_REITER,
                           erzeugeFrame( MODIFIKATIONEN_FRAME,
                                         Util.erzeugeScrollPane( modifikationen ) ) );

        tabbedPane.addTab( VORAUSSETZUNGEN_REITER,
                           erzeugeFrame( VORAUSSETZUNGEN_FRAME, voraussetzungen ) );

        tabbedPane.addTab( BESONDERER_BESITZ_REITER,
                erzeugeFrame( BESONDERER_BESITZ_FRAME,
                              Util.erzeugeScrollPane( besondererBesitz ) ) );

        tabbedPane.addTab( TALENTE_REITER,
                           erzeugeFrame( TALENTE_FRAME, talentboni ) );

        tabbedPane.addTab( LEITTALENTE_REITER,
                           erzeugeFrame( LEITTALENTE_FRAME,
                                         Util.erzeugeScrollPane( leittalente ) ) );

        tabbedPane.addTab( VORTEILE_REITER,
                           erzeugeFrame( VORTEILE_FRAME, vorteile ) );

        tabbedPane.addTab( NACHTEILE_REITER,
                           erzeugeFrame( NACHTEILE_FRAME, nachteile ) );

        tabbedPane.addTab( EMPF_VOR_NACHTEILE_REITER,
                           erzeugeFrame( EMPF_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( empfohlene ) ) );

        tabbedPane.addTab( UNGE_VOR_NACHTEILE_REITER,
                           erzeugeFrame( UNGE_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( ungeeignete ) ) );

        tabbedPane.addTab( SONDERFERTIGKEITEN_REITER,
                           erzeugeFrame( SONDERFERTIGKEITEN_FRAME,
                                         sonderfertigkeiten ) );

        tabbedPane.addTab( VERBILLIGTE_SF_REITER,
                           erzeugeFrame( VERBILLIGTE_SF_FRAME, verbilligteSf ) );

        tabbedPane.addTab( VERBILLIGTE_VARIANTEN_REITER,
                erzeugeFrame( VERBILLIGTE_VARIANTEN_FRAME,
                              verbilligteVarianten ) );

        tabbedPane.addTab( ZAUBER_REITER,
                           erzeugeFrame( ZAUBER_FRAME, zauberboni ) );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START  );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Profession profession ) {

        besondererBesitz.anzeigen( profession );

        diverses.anzeigen( profession );

        empfohlene.setGewaehlteElemente( profession.getEmpfohleneVorNachteile() );

        leittalente.anzeigen( profession );

        modifikationen.anzeigen( profession );

        nachteile.anzeigen( profession.getNachteile() );

        sonderfertigkeiten.anzeigen( profession.getSonderfertigkeiten() );

        talentboni.anzeigen( profession );

        ungeeignete.anzeigen( profession.getUngeeigneteVorNachteile() );

        verbilligteSf.anzeigen( profession );

        verbilligteVarianten.anzeigen( profession );

        voraussetzungen.anzeigen( profession );

        vorteile.anzeigen( profession.getVorteile() );

        zauberboni.anzeigen( profession );
    }

    private SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JComponent inhalt ) {

        final SimpleInternalFrame frame =
            new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;
    }

    /** @return die fuer die Profession gewaehlte Id. */
    public String getProfessionId() {
        return diverses.getProfessionId();
    }

    /**
     * @return den Namen, den der Benutzer fuer die Profession angegeben hat.
     */
    public String getProfessionName() {
        return diverses.getProfessionName();
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Profession
     * eingeben kann.
     */
    private void neu() {

        besondererBesitz.reset();

        diverses.reset();

        empfohlene.reset();

        leittalente.reset();

        modifikationen.reset();

        nachteile.reset();

        sonderfertigkeiten.reset();

        talentboni.reset();

        ungeeignete.reset();

        verbilligteSf.reset();

        verbilligteVarianten.reset();

        voraussetzungen.reset();

        vorteile.reset();

        zauberboni.reset();
    }

    public final void uebertrageDaten( final Profession profession ) {

        profession.setBesondererBesitz( besondererBesitz.getBesondererBesitz());

        diverses.uebertrageDaten( profession );

        profession.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        leittalente.uebertrageDaten( profession );

        modifikationen.uebertrageDaten( profession );

        profession.setNachteile( nachteile.getBoni() );

        profession.setSonderfertigkeiten( sonderfertigkeiten.getBoni() );

        profession.setTalente( talentboni.getBoni() );

        profession.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        profession.setVerbilligte(
                verbilligteSf.getVerbilligteSonderfertigkeiten() );

        profession.setVerbilligteVariante(
                verbilligteVarianten.getVerbilligteVarianten() );

        profession.setVoraussetzungen( voraussetzungen.getVoraussetzungen() );

        profession.setVorteile( vorteile.getBoni() );

        profession.setZauberboni( zauberboni.getBoni() );
    }

}
