/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.SchildTyp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class PanelMetaSchilde extends JPanel {

    private static final String BF =
        I18n.i18n( "PanelMetaSchilde.Bf" );

    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaSchilde.Entfernen" );

    private static final String HINWEIS_DATENQUELLE =
        I18n.i18n( "PanelMetaSchilde.HinweisDatenquelle" );

    private static final String ID =
        I18n.i18n( "PanelMetaSchilde.Id" );

    private static final String INI =
        I18n.i18n( "PanelMetaSchilde.Ini");

    private static final String NAME =
        I18n.i18n( "PanelMetaSchilde.Name" );

    private static final String TYP =
        I18n.i18n( "PanelMetaSchilde.Typ" );

    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaSchilde.Uebernehmen");

    private static final String WM =
        I18n.i18n( "PanelMetaSchilde.Wm" );

    /** Der Bruchfaktor. */
    private final JSpinner bf;

    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Initiativemodifikator. */
    private final JSpinner ini;

    /** Der Name. */
    private final JTextField name;

    /** Die Liste der Schilde. */
    private final JList schilde;

    /** Da werden die Schilde gespeichert. */
    private final Datenspeicher speicher;

    /** Der Typ des Schildes. (Deckung im Fernkampf) */
    private final JComboBox typ;

    /**
     * Der Knopf mit dem der eingegebene Schild in die Liste der Schilde
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    /** Hier kann der Waffenmodifikator eingegeben werden. */
    private final JTextField wm;

    /** Zum ueberpruefen der Eingabe ins Textfeld {@link #wm}. */
    private final Pattern wmPattern;

    public PanelMetaSchilde() {

        bf = Util.erzeugeSpinner( 0, -9, 99, 1 );

        hinweisGezeigt = false;

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        ini = Util.erzeugeSpinner( 0, -9, 9, 1 );

        name = new JTextField();

        speicher = Datenspeicher.getInstanz();

        final Vector< Schild > vector =
            new Vector< Schild >( speicher.getSchilde() );

        schilde = new JList( vector );
        schilde.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        schilde.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });

        typ = new JComboBox( new Vector< Object >( SchildTyp.getTypen() ) );

        wm = new JTextField();
        wm.setBackground( Util.FEHLERFARBE );
        wm.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        wmPattern = Pattern.compile( "[+-]?\\d+/[+-]?\\d+" );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final int index = schilde.getSelectedIndex();

        if ( 0 <= index ) {

            final Schild schild = (Schild) schilde.getSelectedValue();

            bf.setValue( schild.getBf() );

            id.setText( schild.getId().replace( "Schild.", "" ) );

            ini.setValue( schild.getIni() );

            name.setText( schild.getName() );

            typ.setSelectedItem( schild.getTyp() );

            wm.setText( String.format(
                    "%1$s/%2$s",
                    schild.getWmAt(),
                    schild.getWmPa() ) );
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        final String spalten = "7dlu, 200px:G(0.2), 4dlu, P, 4dlu, P, P:G(0.8), 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
            "4dlu, P, 4dlu, P, P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( schilde ), cc.xywh( 2, 2, 1, 14 ) );

        add( entfernen, cc.xy( 2, 17 ) );

        int y = 0;

        add( new JLabel( NAME ), cc.xy ( 4, y+=2 ) );
        add( name,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( ID ), cc.xy ( 4, y+=2 ) );
        add( id,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( WM ), cc.xy ( 4, y+=2 ) );
        add( wm,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( INI ), cc.xy( 4, y+=2 ) );
        add( ini,               cc.xy( 6, y ) );

        add( new JLabel( BF ), cc.xy( 4, y+=2 ) );
        add( bf,               cc.xy( 6, y ) );

        add( new JLabel( TYP ), cc.xy( 4, y+=2 ) );
        add( typ,               cc.xy( 6, y ) );

        add( buttonpanel, cc.xyw( 4, y+=2, 4 ) );
    }

    /**
     * Enternt die in der Liste {@link #ruestungen} gewaehlte Ruestung aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final int index = schilde.getSelectedIndex();

        if ( 0 <= index ) {

            final Schild schild =
                (Schild) schilde.getSelectedValue();

            speicher.entfernen( schild );

            final List< Schild > liste = speicher.getSchilde();

            final Vector< Schild > vector =
                new Vector< Schild >( liste );

            schilde.setListData( vector );

            speichern();
        }
    }

    /**
     * Prueft die Eingabe im Panel auf Korrektheit und setzt den
     * Uebernehmen-Knopf auf aktiv oder deaktiv.
     *
     * @return  {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            ergebnis = false;

        } else {

            id.setBackground( name.getBackground() );
        }

        final Matcher matcher = wmPattern.matcher( wm.getText().trim() );

        if ( matcher.matches() ) {

            wm.setBackground( name.getBackground() );

        } else {

            wm.setBackground( Util.FEHLERFARBE );

            ergebnis = false;
        }

        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }

    /**
     * Speichert die Liste der Schilde auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereSchilde();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( this, HINWEIS_DATENQUELLE );

            hinweisGezeigt = true;
        }

    }

    /**
     * Wandelt einen String, der eine Zahl mit Vorzeichen enthaelt in einen Int
     * Wert um.
     *
     * @param s
     *            die Zahl als String.
     */
    private final int toInt( String s ) {

        if ( s.charAt( 0 ) == '+' ) {

            s = s.substring( 1 );
        }

        return Integer.valueOf( s );
    }

    /**
     * Uebernimmt die im Panel eingegebenen Ruestung in die Liste der
     * Ruestungen.
     */
    private final void uebernehmen() {

        String idString = id.getText().trim();

        idString = "Schild." + idString;

        final Schild schild = speicher.getSchild( idString );

        schild.setBf( (Integer) bf.getValue() );

        schild.setIni( (Integer) ini.getValue() );

        schild.setName( name.getText().trim() );

        schild.setTyp( (SchildTyp) typ.getSelectedItem() );

        final String[] wmArray = wm.getText().trim().split( "/" );

        schild.setWmAt( toInt( wmArray[ 0 ] ) );

        schild.setWmPa( toInt( wmArray[ 1 ] ) );

        speicher.einfuegen( schild );

        speichern();

        final List< Schild > liste = speicher.getSchilde();

        final Vector< Schild > vector = new Vector< Schild >( liste );

        schilde.setListData( vector );
    }
}
