/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFestAlteSprache;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 *
 * @author Bernhard Jung
 *
 */
class PanelMetaTalentbonusFestAlteSprache extends JPanel {

    /** Die Beschreibung des Bonus, der erzeugt wird. */
    private static final String BESCHREIBUNG =
        I18n.i18n( "PanelMetaTalentbonusFestAlteSprache.Beschreibung" );

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Stufenboni gewaehlt
     * werden.
     */
    private static final String BONI =
        I18n.i18n( "PanelMetaTalentbonusFest.Boni" );

    /** Der Text neben dem einen Bonus. */
    private static final String BONUS =
        I18n.i18n( "PanelMetaTalentbonusFestAlteSprache.Bonus" );

    /** Die Ueberschrift fuer die Talente. */
    private static final String TALENTE =
        I18n.i18n( "PanelMetaTalentbonusFest.Talente" );

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaTalentbonusFest.Uebernehmen" );

    /** Die Beschreibung des Bonus. */
    private final JTextArea beschreibung;

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus1;

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus2;

    /** Die Liste von Talentboni, in die man, ueber dieses Panel, neue Talentboni einfuegt. */
    private final AenderbaresListModel< Talentbonus > talentboni;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;

    PanelMetaTalentbonusFestAlteSprache(
            final AenderbaresListModel< Talentbonus > dieTalentboni ) {

        beschreibung = new JTextArea( BESCHREIBUNG );
        beschreibung.setLineWrap( true );
        beschreibung.setWrapStyleWord( true );
        beschreibung.setEditable( false );

        bonus1 = Util.erzeugeSpinner( 0, 0, 99, 1 );

        bonus2 = Util.erzeugeSpinner( 0, 0, 99, 1 );

        talentboni = dieTalentboni;

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final List< Integer > boni = Arrays.asList(
                        (Integer) bonus1.getValue(),
                        (Integer) bonus2.getValue() );

                final TalentbonusFest bonus =
                    new TalentbonusFestAlteSprache( boni );

                talentboni.add( bonus );
            }
        });

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final String spalten = "P, 4dlu, 120px:G";

        final String zeilen = "P, 4dlu, F:60px:G, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        final DefaultComponentFactory factory =
            DefaultComponentFactory.getInstance();

        builder.append( factory.createSeparator( TALENTE ), 3 );

        builder.nextLine( 2 );

        builder.append( erzeugeScrollPane( beschreibung, 120, 60 ), 3 );

        builder.nextLine( 2 );

        builder.append( factory.createSeparator( BONI ), 3 );

        builder.nextLine( 2 );

        builder.append( BONUS, bonus1 );

        builder.nextLine( 2 );

        builder.append( BONUS, bonus2 );

        builder.nextLine( 2 );

        builder.append( uebernehmen, 3 );
    }

    private JScrollPane erzeugeScrollPane(
            final JComponent component,
            final int breite,
            final int hoehe ) {

        final JScrollPane scrollPane = new JScrollPane( component );

        scrollPane.setPreferredSize( new Dimension( breite, hoehe ) );

        return scrollPane;
    }
}
