/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

public class PanelMetaTalentbonusFrei extends JPanel {

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Punkte gewaehlt
     * werden, die man auf die Talente verteilen darf.
     */
    private static final String BONUS =
        I18n.i18n( "PanelMetaTalentbonusFrei.Bonus" );

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Talente gewaehlt
     * werden.
     */
    private static final String TALENTE =
        I18n.i18n( "PanelMetaTalentbonusFest.Talente" );

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaTalentbonusFrei.Uebernehmen" );

    /**
     * Der Text neben dem Spinner in dem man die verteilbaren Talentpunkte
     * einstellt.
     */
    private static final String VERTEILEN =
        I18n.i18n( "PanelMetaTalentbonusFrei.Verteilen" );

    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Talente. */
    private final JList gewaehlteTalenteListe;

    /** Das ListModel fuer die Liste der gewaehlten Talente. */
    private final AenderbaresListModel< Talent > gewaehlteTalenteModel;

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner punkte;

    /** Eine Liste aller Talente. */
    private final JList talente;

    /** Die Liste von Talentboni, in die man, ueber dieses Panel, neue Talentboni einfuegt. */
    private final AenderbaresListModel< Talentbonus > talentboni;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;

    PanelMetaTalentbonusFrei(
            final AenderbaresListModel< Talentbonus > dieTalentboni ) {

        gewaehlteTalenteModel = new AenderbaresListModel< Talent >();

        gewaehlteTalenteListe = new JList( gewaehlteTalenteModel );
        gewaehlteTalenteListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        gewaehlteTalenteListe.addListSelectionListener(
                new ListSelectionListener(){
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = gewaehlteTalenteListe.getSelectedIndex();
                        if ( index >= 0 ) {
                            gewaehlteTalenteModel.remove( index );
                            pruefen();
                        }
                    }
                });

        punkte = Util.erzeugeSpinner( 1, 1, 99, 1 );

        final Vector< Talent > talentliste = new Vector< Talent >();
        for ( final Talentgruppe gruppe: Talentgruppe.getTalentgruppenAlle() ) {
            talentliste.addAll( gruppe );
        }
        talente = new JList( talentliste );
        talente.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        talente.addMouseListener( new MouseAdapter() {
            @Override
            public void mouseClicked( final MouseEvent e ) {
                final Object talent = talente.getSelectedValue();
                if ( null != talent ) {
                    if ( ! gewaehlteTalenteModel.contains( (Talent) talent ) ) {
                        gewaehlteTalenteModel.add( (Talent) talent );
                        pruefen();
                    }
                }
            }
        } );

        talentboni = dieTalentboni;

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final Talentbonus bonus = new TalentbonusFrei(
                        (Integer) punkte.getValue(),
                        gewaehlteTalenteModel.getElemente() );

                talentboni.add( bonus );

                gewaehlteTalenteModel.clear();

                pruefen();
            }
        });
        uebernehmen.setEnabled( false );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JPanel spinnerPanel = new JPanel(
                new FormLayout( "P, 4dlu, P", "P" ) );

        final CellConstraints cc = new CellConstraints();

        spinnerPanel.add( new JLabel( VERTEILEN ), cc.xy( 1, 1 ) );

        spinnerPanel.add( punkte, cc.xy( 3, 1 ) );

        final String spalten = "7dlu, F:P:G, 4dlu, F:P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, F:P:G, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        add( erzeugeScrollPane( talente, 150, 200 ), cc.xywh( 2, 2, 1, 7 ) );

        add( uebernehmen, cc.xy( 2, 10 ) );

        final DefaultComponentFactory factory =
            DefaultComponentFactory.getInstance();

        add( factory.createSeparator( TALENTE ), cc.xy( 4, 2 ) );

        add( erzeugeScrollPane( gewaehlteTalenteListe, 150, 100 ),
             cc.xy( 4, 4 ) );

        add( factory.createSeparator( BONUS ), cc.xy( 4, 6 ) );

        add( spinnerPanel, cc.xy( 4, 8 ) );
    }

    private JScrollPane erzeugeScrollPane(
            final JComponent component,
            final int breite,
            final int hoehe ) {

        final JScrollPane scrollPane = new JScrollPane( component );

        scrollPane.setPreferredSize( new Dimension( breite, hoehe ) );

        return scrollPane;
    }

    private void pruefen() {

        final int anzahlTalente = gewaehlteTalenteModel.getSize();

        uebernehmen.setEnabled( anzahlTalente > 1 );
    }

    /** Loescht die gewaehlten Talente. */
    public void reset() {
        gewaehlteTalenteModel.clear();
    }
}
