/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungMindeststufe;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungSo;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Ein Panel fuer die Voraussetzungen von Professionen.
 *
 * @author Bernhard Jung
 *
 */
class PanelMetaVoraussetzungen
extends JPanel
implements ActionListener {

    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaVoraussetzungenEigenschaften.Entfernen" );

    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaVoraussetzungenEigenschaften.Uebernehmen" );

    private static final String VORAUSSETZUNG =
        I18n.i18n( "PanelMetaVoraussetzungenEigenschaften.Voraussetzung" );

    private final Eigenschaft[] eigenschaften;

    private final List< JCheckBox > checkBoxen;

    private final JSpinner mindeststufe;

    /** Der Spinner mit dem die SO Mindeststufe eingestellt wird. */
    private final JSpinner soMindeststufe;

    /** Der Spinner mit dem die SO Hoechststufe eingestellt wird. */
    private final JSpinner soHoechststufe;

    private final JButton uebernehmen;

    private final AenderbaresListModel< Voraussetzung > voraussetzungenModel;

    PanelMetaVoraussetzungen() {

        eigenschaften = Eigenschaft.getEigenschaften();

        checkBoxen =
            new ArrayList< JCheckBox >( eigenschaften.length );

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            final JCheckBox checkBox = new JCheckBox( eigenschaft.getName() );

            checkBox.addActionListener( this );

            checkBoxen.add( checkBox );
        }

        mindeststufe = Util.erzeugeSpinner( 11, 0, 99, 1 );

        soMindeststufe = Util.erzeugeSpinner( 1, 1, 99, 1 );

        soHoechststufe = Util.erzeugeSpinner( 13, 1, 99, 1 );

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        voraussetzungenModel = new AenderbaresListModel< Voraussetzung >();

        final JList voraussetzungenListe = new JList( voraussetzungenModel );
        voraussetzungenListe.setSelectionMode(
                ListSelectionModel.SINGLE_SELECTION );

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final int index = voraussetzungenListe.getSelectedIndex();

                if ( index >= 0 ) {

                    voraussetzungenModel.remove( index );
                }
            }
        });

        final FormLayout layout = new FormLayout(
                "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, P:G, 4dlu, P, 4dlu, P, 7dlu",
                "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, " +
                      "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu");

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JLabel( Eigenschaft.SO.getName() ), cc.xy( 2, 2 ) );
        add( soMindeststufe, cc.xy( 4, 2 ) );
        add( new JLabel( "-" ), cc.xy( 6, 2 ) );
        add( soHoechststufe, cc.xy( 8, 2 ) );

        add( new JScrollPane( voraussetzungenListe ), cc.xywh( 2, 4, 8, 15 ) );

        add( entfernen, cc.xyw( 2, 20, 8 ) );

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            final int y = 2 + ( 2 * i );

            add( checkBoxen.get( i ), cc.xyw( 11, y, 3 ) );
        }

        add( new JLabel( VORAUSSETZUNG ), cc.xy( 11, 18 ) );
        add( mindeststufe, cc.xy( 13, 18 ) );

        add( uebernehmen, cc.xyw( 11, 20, 3 ) );
    }

    /**
     * Wird aufgerufen, wenn eine Checkbox angeklickt wird.
     */
    public void actionPerformed( final ActionEvent e ) {

        boolean eigenschaftGewaehlt = false;

        for ( final JCheckBox checkBox : checkBoxen ) {

            if ( checkBox.isSelected() ) {

                eigenschaftGewaehlt = true;
                break;
            }
        }

        uebernehmen.setEnabled( eigenschaftGewaehlt );
    }

    /**
     * Zeigt die Eigenschaftsvoraussetzungen der Profession an.
     *
     * @param profession
     *            deren Voraussetzungen angezeigt werden sollen.
     */
    public final void anzeigen( final Profession profession ) {

        voraussetzungenModel.clear();

        final List< Voraussetzung > voraussetzungen =
            profession.getVoraussetzungen();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            if ( voraussetzung instanceof VoraussetzungMindeststufe ) {

                voraussetzungenModel.add( voraussetzung );

            } else if ( voraussetzung instanceof VoraussetzungSo ) {

                anzeigen( (VoraussetzungSo) voraussetzung );
            }
        }
    }

    /**
     * Stellt die Spinner entsprechend der Voraussetzung ein.
     *
     * @param voraussetzung
     *            die angezeigt werden soll.
     */
    private void anzeigen( final VoraussetzungSo voraussetzung ) {

        soMindeststufe.setValue( voraussetzung.getMindeststufe() );

        soHoechststufe.setValue( voraussetzung.getHoechststufe() );
    }


    /**
     * @return die gewaehlten Voraussetzungen.
     */
    public final List< Voraussetzung > getVoraussetzungen() {

        final List< Voraussetzung > voraussetzungen =
            voraussetzungenModel.getElemente();

        voraussetzungen.add( new VoraussetzungSo(
                (Integer) soMindeststufe.getValue(),
                (Integer) soHoechststufe.getValue() ) );

        return voraussetzungen;
    }

    /** Loescht alle Eintraege im Panel. */
    public void reset() {

        for ( final JCheckBox checkBox : checkBoxen ) {

            checkBox.setSelected( false );
        }

        uebernehmen.setEnabled( false );

        voraussetzungenModel.clear();
    }

    /**
     * Uebernimmt die eingestellte Voraussetzung in die Liste der
     * Voraussetzungen.
     */
    public final void uebernehmen() {

        final List< Eigenschaft > liste = new ArrayList< Eigenschaft >();

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            final JCheckBox checkBox = checkBoxen.get( i );

            if ( checkBox.isSelected() ) {

                checkBox.setSelected( false );

                liste.add( eigenschaften[ i ] );
            }
        }

        uebernehmen.setEnabled( false );

        voraussetzungenModel.add( new VoraussetzungMindeststufe(
                (Integer) mindeststufe.getValue(),
                liste.toArray( new Eigenschaft[ liste.size() ] ) ) );
    }

}
