/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.basiswerte.BasiswertWert;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

public class PanelBasiswerteSteigern extends JPanel implements Observer {

    /** Die Ueberschrift fuer die Spalte mit den AP-Kosten. */
    private static final String AP =
        I18n.i18n( "PanelBasiswerteSteigern.Ap" );

    /** Die Ueberschrift fuer die Spalte mit den Basiswerten. */
    private static final String BASISWERT =
        I18n.i18n( "PanelBasiswerteSteigern.Basiswert" );

    /** Die Ueberschrift fuer die Spalte mit der Gesamtstufe der Basiswerte. */
    private static final String GESAMT =
        I18n.i18n( "PanelBasiswerteSteigern.Gesamtstufe" );

    /**
     * Die Ueberschrift fuer die Spalte mit den Checkboxen fuer die Spezielle
     * Erfahrung.
     */
    private static final String SPEZIELLE_ERFAHRUNG =
        I18n.i18n( "PanelBasiswerteSteigern.SpezielleErfahrung" );

    /** Die Ueberschrift fuer die Spalte mit den zugekauften Punkten. */
    private static final String ZUKAUF =
        I18n.i18n( "PanelBasiswerteSteigern.Zukauf" );

    /**
     * Die Action die zum setzen/entfernen einer Speziellen Erfahrung verwendet
     * wird.
     */
    private final Action actionSpezielleErfahrung;

    /** Die Action die zum Steigern eines Basiswerts verwendet wird. */
    private final ChangeListener actionZukauf;

    /** Die Label die die Steigerungskosten anzeigen. */
    private final List< JLabel > ap;

    /** Die Basiswerte die angezeigt werden. */
    private final Basiswert[] basiswerte;

    /** Die Label die die Namen der Basiswerte anzeigen. */
    private final List< JLabel > basiswerteLabel;

    /** Die Label die die Gesamtstufe der Basiswerte anzeigen. */
    private final List< JLabel > gesamt;

    /** Der Knopf mit dem einen grosse Meditation durchgefuehrt werden kann. */
    private final JButton grosseMeditation;

    /** Der Held dessen Werte angezeigt und manipuliert werden. */
    private Held held;

    /**
     * Die Checkbox mit der einen Spezielle Erfahrung fuer die Ausdauer
     * angegeben werden kann.
     */
    private final JCheckBox spezielleErfahrungAu;

    /**
     * Die Checkbox mit der einen Spezielle Erfahrung fuer die Lebenenergie
     * angegeben werden kann.
     */
    private final JCheckBox spezielleErfahrungLe;

    /** Zeigt die Stufe des Basiswerts an, ohne die zugekauften Punkte. */
    private final List< JLabel > stufenOhneZukauf;

    /** Zeigen die zum Basiswert zugekauften Punkte an. */
    private final List< JSpinner > zukauf;

    public PanelBasiswerteSteigern() {

        actionSpezielleErfahrung = new AbstractAction() {
            public void actionPerformed( final ActionEvent e ) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                held.getWert( basiswerte[ index ] ).setSpezielleErfahrung(
                        checkBox.isSelected() );
            }
        };

        actionZukauf = new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                final JSpinner spinner= (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                final int punkte = (Integer) spinner.getValue();

                final BasiswertWert wert = held.getWert( basiswerte[ index ] );

                wert.setZukauf( punkte );
            }
        };

        basiswerte = Basiswert.getBasiswerte();

        ap = new ArrayList< JLabel >( basiswerte.length );

        basiswerteLabel = new ArrayList< JLabel >( basiswerte.length );

        gesamt = new ArrayList< JLabel >( basiswerte.length );

        grosseMeditation = new JButton(
                Sonderfertigkeit.GROSSE_MEDITATION.getName() );
        grosseMeditation.addActionListener( new ActionGrosseMeditation() );

        spezielleErfahrungAu = new JCheckBox( actionSpezielleErfahrung );
        spezielleErfahrungAu.setName(
                String.valueOf( Basiswert.AU.getIndex() ) );
        spezielleErfahrungAu.setToolTipText(
                Lernmethode.SPEZIELLE_ERFAHRUNG.toString() );

        spezielleErfahrungLe = new JCheckBox( actionSpezielleErfahrung );
        spezielleErfahrungLe.setName(
                String.valueOf( Basiswert.LE.getIndex() ) );
        spezielleErfahrungLe.setToolTipText(
                Lernmethode.SPEZIELLE_ERFAHRUNG.toString() );

        stufenOhneZukauf = new ArrayList< JLabel >( basiswerte.length );

        zukauf = new ArrayList< JSpinner >( basiswerte.length );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Erzeugt die Elemente der Benutzeroberflaeche und ordnet sie auf dem Panel
     * an.
     */
    private final void aufbauenBenutzeroberflaeche() {

        // Die einzelnen Element erzeugen. Es werden ein paar Elemente zu viel
        // erzeugt.
        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            ap.add( Util.erzeugeLabelRechtsbuendig() );

            basiswerteLabel.add( new JLabel( basiswerte[ i ].getName() ) );

            gesamt.add( Util.erzeugeLabelRechtsbuendig() );

            stufenOhneZukauf.add( Util.erzeugeLabelRechtsbuendig() );

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );
            spinner.setName( String.valueOf( i ) );
            spinner.addChangeListener( actionZukauf );
            zukauf.add( spinner );
        }

        final FormLayout layout = new FormLayout(
          //Basiswert   stufe      +      Zukauf    AP        =      Gesamt    SE
                "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();
        builder.setRowGroupingEnabled( true );

        builder.append( Util.erzeugeUeberschrift( BASISWERT ) );
        builder.append( "" );
        builder.append( "" );
        builder.append( Util.erzeugeUeberschrift( ZUKAUF ) );
        builder.append( Util.erzeugeUeberschrift( AP ) );
        builder.append( "" );
        builder.append( Util.erzeugeUeberschrift( GESAMT ) );
        builder.append( Util.erzeugeUeberschrift( SPEZIELLE_ERFAHRUNG ) );

        for ( int i = 0, n = basiswerte.length; i < n ; ++i ) {

            builder.append( basiswerteLabel.get( i ) );

            builder.append( stufenOhneZukauf.get( i ) );

            if ( basiswerte[ i ].istSteigerbar() ) {

                builder.append( "+" );

                builder.append( zukauf.get( i ) );

                builder.append( ap.get( i ) );

                builder.append( "=" );

                builder.append( gesamt.get( i ) );

                if ( basiswerte[ i ].equals( Basiswert.AU ) ) {

                    builder.append( spezielleErfahrungAu );

                } else if ( basiswerte[ i ].equals( Basiswert.LE ) ) {

                    builder.append( spezielleErfahrungLe );

                } else if ( basiswerte[ i ].equals( Basiswert.AE ) ) {

                    builder.append( grosseMeditation );

                } else {

                    builder.nextLine( 1 );

                }

            } else {

                builder.nextLine( 1 );
            }
        }

        setLayout( new BorderLayout() );

        add( builder.getPanel(), BorderLayout.CENTER );
    }

    /**
     * @param derHeld dessen Werte angezeigt und manipuliert werden.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        update( null, null );
    }

    /**
     * Aktualisiert eine Zeile in der Benutzeroberflaeche.
     *
     * @param index des Basiswertes, dessen Zeile aktualisiert werden soll.
     */
    private final void update( final int index ) {

        final Basiswert basiswert = basiswerte[ index ];

        final BasiswertWert wert = held.getWert( basiswert );

        final KonfliktBeschreibung beschreibung = wert.pruefen();

        if ( beschreibung.enthaeltKonflikt() ) {

            basiswerteLabel.get( index ).setForeground( Util.FEHLERFARBE );
            basiswerteLabel.get( index ).setToolTipText(
                    beschreibung.toHtml() );

        } else {

            basiswerteLabel.get( index ).setForeground( getForeground() );
            basiswerteLabel.get( index ).setToolTipText( null );

        }

        final int stufe = wert.getStufe();

        final int z = wert.getZukauf();

        stufenOhneZukauf.get( index ).setText( String.valueOf( stufe - z ) );

        if ( basiswert.istSteigerbar() ) {

            zukauf.get( index ).setValue( z );

            ap.get( index ).setText( "(" + wert.getSteigerungskosten() + ")" );

            gesamt.get( index ).setText( String.valueOf( stufe ) );

            if ( basiswert.equals( Basiswert.AE ) ) {

                zukauf.get( index ).setEnabled( held.istZauberer() );

                grosseMeditation.setEnabled(
                        held.hat( Sonderfertigkeit.GROSSE_MEDITATION ) );

            } else if ( basiswert.equals( Basiswert.LE ) ) {

                spezielleErfahrungLe.setSelected( wert.hatSpezielleErfahrung());

            } else if ( basiswert.equals( Basiswert.AU ) ) {

                spezielleErfahrungAu.setSelected( wert.hatSpezielleErfahrung());
            }

        }
    }

    /**
     * Wird aufgerufen, wenn eine Aenderung am Held durchgefuehrt wird.
     *
     * @param o ungenutzt
     * @param arg ungenutzt
     */
    public final void update( final Observable o, final Object arg ) {

        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            update( i );
        }
    }

    private final class ActionGrosseMeditation extends AbstractAction {

        private final int AP_KOSTEN_GROSSE_MEDITATION = 400;

        private final String GROSSE_MEDITATION =
            I18n.i18n( "PanelBasiswerteSteigern.GrosseMeditation" );

        ActionGrosseMeditation() {
            super( Sonderfertigkeit.GROSSE_MEDITATION.toString() );
        }

        public void actionPerformed( final ActionEvent e ) {

            final String bonusString = JOptionPane.showInputDialog(
                    PanelBasiswerteSteigern.this,
                    GROSSE_MEDITATION );

            try {

                final int bonus = Integer.valueOf( bonusString );

                held.getWert( Basiswert.AE ).addiereBonuspunkte( bonus );

                held.addiereApEingesetzt( AP_KOSTEN_GROSSE_MEDITATION );

            } catch ( final NumberFormatException ex ) {
                // ignorieren
            }
        }
    }
}
