/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Berechnet fuer die Talente die durch Schlangenmensch verbilligt werden
 * den Steigerungsfaktor. Das gilt nicht fuer das Talent Ringen. Ringen wird als
 * Sonderfall in {@link KostenKampftechnik} abgehandelt.
 *
 * @author Bernhard Jung
 *
 */
class KostenSchlangenmensch extends KostenStandard {

    /** @return Begabung Talent, Begabung Talentgruppe, Elfische Weltsicht. */
    @Override
    public Collection<Object> getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( VorNachteil.SCHLANGENMENSCH );

        return abhaengigkeiten;
    }

    /** {@inheritDoc} */
    @Override
    public int getSteigerungsspalte(
            final Held held,
            final Talent talent,
            final int stufe, Lernmethode lernmethode ) {

        int steigerungsfaktor = super.getSteigerungsspalte(held, talent, stufe, lernmethode);

        if ( ( ! begabungWirkt ) && held.hat( VorNachteil.SCHLANGENMENSCH ) ) {

            --steigerungsfaktor;
        }

        return steigerungsfaktor;
    }
}
