/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Eine Voraussetzung bei der geprueft wird ob der Held eine Schrift auf
 * einer Stufe >= 4 besitzt. 
 * 
 * @author Bernhard Jung
 */
class VoraussetzungAktivierungSchrift extends Voraussetzung {
    
    /**
     * Die Meldung die ausgegeben wird, wenn die Voraussetzung nicht erfuellt
     * wird.
     */
    private final String beschreibungString;
    
    VoraussetzungAktivierungSchrift() {
        beschreibungString = I18n.i18n( "VoraussetzungAktivierungSchrift" );
    }
    
    /**
     * @return eine Liste aller Schriften.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return new ArrayList< Object >( Talentgruppe.SCHRIFTEN );
    }
    
    public KonfliktBeschreibung pruefen( 
            final Held held, 
            final Talent talent ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        boolean beherrschtSchrift = false;
        
        for ( final Talent schrift : Talentgruppe.SCHRIFTEN ) {
            
            if ( held.hat( schrift ) 
                    && ( held.getStufe( schrift ) >= 4 ) ) {
                
                beherrschtSchrift = true;
                break;
            }
        }
        
        if ( ! beherrschtSchrift ) {
            
            beschreibung.einfuegen( beschreibungString );
        }
        
        return beschreibung;
    }

}
