/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen.VoraussetzungNichtKombinierbar;

/**
 * Fuer ein Talent das nicht mit einem bestimmten Vor-/Nachteil kombiniert
 * werden kann.
 * 
 * @author Bernhard Jung
 *
 */
class VoraussetzungNichtVorNachteil extends Voraussetzung {

    /** Der Vor-/Nachteil den der Held nicht besitzen darf. */
    private final VorNachteil vorNachteil;
    
    /**
     * @param derVorNachteil den der Held nicht besitzen darf.
     */
    VoraussetzungNichtVorNachteil( final VorNachteil derVorNachteil ) {
        vorNachteil = derVorNachteil;
    }
    
    /**
     * @return den Vor-/Nachteil.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorNachteil );
    }
    
    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( 
            final Held held, 
            final Talent talent ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( held.hat( vorNachteil ) ) {
            
            beschreibung.einfuegen( String.format(
                    VoraussetzungNichtKombinierbar.NICHT_KOMBINIERBAR,
                    vorNachteil ) );
        }
        
        return beschreibung;
    }

}
