/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Berechnet die GP-Kosten fuer eine Vor-/Nachteil.
 *
 * @author Bernhard Jung
 */
public interface Kosten {

    /** Berechnet die Kosten fuer den Nachteil Albino. */
    public static final Kosten ALBINO = new KostenAlbino();

    /**
     * Astraler Block bringt fuer Vollzauberer 10 GP und fuer Viertelzauberer
     * 5 GP.
     */
    public static final Kosten ASTRALER_BLOCK = new KostenZV( -10, -5 );

    /** Bei Balance wird die Sonderfertigkeit Standfest angerechnet. */
    public static final Kosten BALANCE =
        new KostenSonderfertigkeit( 10, Sonderfertigkeit.STANDFEST );

    /** Bei Begabung fuer Talent haengen die Kosten vom Talent ab. */
    public static final Kosten BEGABUNG_TALENT = new KostenBegabungTalent();

    /** Bei Begabung fuer Talent haengen die Kosten vom Talent ab. */
    public static final Kosten BEGABUNG_TALENTGRUPPE =
        new KostenBegabungTalentgruppe();

    /** Die Kosten fuer den Vorteil BesondererBesitz. */
    public static final Kosten BESONDERER_BESITZ =
        new KostenBesondererBesitz();

    /** Elfen und Magier muessen fuer Gutes Gedaechtnis mehr bezahlen. */
    public static final Kosten GUTES_GEDAECHTNIS = new KostenGutesGedaechtnis();

    /** Fuer Vorteile die 1 GP kosten. */
    public static final Kosten GP_1 = new KostenFest( 1 );

    /** Fuer Vorteile die 3 GP kosten. */
    public static final Kosten GP_3 = new KostenFest( 3 );

    /** Fuer Vorteile die 4 GP kosten. */
    public static final Kosten GP_4 = new KostenFest( 4 );

    /** Fuer Vorteile die 5 GP kosten. */
    public static final Kosten GP_5 = new KostenFest( 5 );

    /** Fuer Vorteile die 7 GP kosten. */
    public static final Kosten GP_7 = new KostenFest( 7 );

    /** Fuer Vorteile die 8 GP kosten. */
    public static final Kosten GP_8 = new KostenFest( 8 );

    /** Fuer Vorteile die 10 GP kosten. */
    public static final Kosten GP_10 = new KostenFest( 10 );

    /** Fuer Vorteile die 12 GP kosten. */
    public static final Kosten GP_12 = new KostenFest( 12 );

    /** Fuer Vorteile die 15 GP kosten. */
    public static final Kosten GP_15 = new KostenFest( 15 );

    /** Fuer Vorteile die 20 GP kosten. */
    public static final Kosten GP_20 = new KostenFest( 20 );

    /** Fuer Vorteile die 25 GP kosten. */
    public static final Kosten GP_25 = new KostenFest( 25 );

    /** Fuer Vorteile die 35 GP kosten. */
    public static final Kosten GP_35 = new KostenFest( 35 );

    /** Fuer Vorteile die 1 GP pro 1 Punkt kosten. */
    public static final Kosten GP_1_pro_1 = new KostenStufe( 1, 1 );

    /** Fuer Vorteile die 2 GP pro 1 Punkt kosten. */
    public static final Kosten GP_2_pro_1 = new KostenStufe( 2, 1 );

    /** Berechnet die Kosten von Herausragende Balance. */
    public static final Kosten HERAUSRAGENDE_BALANCE =
        new KostenHerausragendeBalance();

    /** Berechnet die Kosten von Herausragende Balance. */
    public static final Kosten HERAUSRAGENDE_EIGENSCHAFT =
        new KostenHerausragendeEigenschaft();

    /** Die Kosten fuer den Vorteil Immunitaet gegen (Einzel)Gift. */
    public static final Kosten IMMUNITAET_GIFT =
        new KostenAngerechneteVariante( 5, VorNachteil.RESISTENZ_GIFT );

    /** Die Kosten fuer den Vorteil Immunitaet gegen (Einzel)Gift. */
    public static final Kosten IMMUNITAET_GIFTART =
        new KostenAngerechneteVariante( 10, VorNachteil.RESISTENZ_GIFTART );

    /**
     * Meisterhandwerk kostet Vollzauberer 5 GP und Viertelzauberer 1 GP.
     */
    public static final Kosten MEISTERHANDWERK = new KostenZV( 5, 1 );

    /** Fuer Schlechte Eigenschaften die 1 GP pro 1 Punkt bringen. */
    public static final Kosten MINUS_1_PRO_1_SE =
        new KostenStufeSchlechteEigenschaft( -1, 1 );

    /** Fuer Nachteile die 1 GP pro 1 Punkt bringen. */
    public static final Kosten MINUS_1_PRO_1 = new KostenStufe( -1, 1 );

    /** Fuer Schlechte Eigenschaften die 1 GP pro 2 Punkte bringen . */
    public static final Kosten MINUS_1_PRO_2_SE =
        new KostenStufeSchlechteEigenschaft( -1, 2 );

    /** Fuer Nachteile die 2 GP pro 1 Punkt bringen. */
    public static final Kosten MINUS_2_PRO_1 = new KostenStufe( -2, 1 );

    /** Fuer Schlechte Eigenschaften die 2 GP pro 1 Punkt bringen. */
    public static final Kosten MINUS_2_PRO_1_SE =
        new KostenStufeSchlechteEigenschaft( -2, 1 );

    /** Fuer Schlechte Eigenschaften die 3 GP pro 2 Punkte bringen. */
    public static final Kosten MINUS_3_PRO_2_SE =
        new KostenStufeSchlechteEigenschaft( -3, 2 );

    /** Fuer Nachteile die 5 GP bringen. */
    public static final Kosten MINUS_5 = new KostenFest( -5 );

    /** Fuer Nachteile die 7 GP bringen. */
    public static final Kosten MINUS_7 = new KostenFest( -7 );

    /** Fuer Nachteile die 10 GP bringen. */
    public static final Kosten MINUS_10 = new KostenFest( -10 );

    /** Fuer Nachteile die 10 GP bringen. */
    public static final Kosten MINUS_12 = new KostenFest( -12 );

    /** Fuer Nachteile die 15 GP bringen. */
    public static final Kosten MINUS_15 = new KostenFest( -15 );

    /** Fuer Nachteile die 25 GP bringen. */
    public static final Kosten MINUS_25 = new KostenFest( -25 );

    /** Fuer Nachteile die 30 GP bringen. */
    public static final Kosten MINUS_30 = new KostenFest( -30 );

    /** Die Kosten fuer den Nachteil Unfaehigkeit fuer [Talent]. */
    public static final Kosten UNFAEHIGKEIT_TALENT =
        new KostenUnfaehigkeitTalent();

    /** Die Kosten fuer den Nachteil Unfaehigkeit fuer [Talentgruppe]. */
    public static final Kosten UNFAEHIGKEIT_TALENTGRUPPE =
        new KostenUnfaehigkeitTalentgruppe();

    /** Die Kosten fuer den Vorteil Verbindungen. */
    public static final Kosten VERBINDUNGEN = new KostenVerbindungen();

    /**
     * Liefert die Kosten die bei einem Nachteil aufzubringen sind, um ihn
     * abzubauen.
     *
     * @param held
     *            der den Nachteil abbauen will.
     * @param wert
     *            der abgebaut werden soll.
     * @return die AP, die bezahlt werden muessen um den Nachteil komplett
     *         abzubauen.
     */
    public int getAbbaukosten(Held held, VorNachteilWert wert);

    /**
     * Die Regelelemente (Vor-/Nachteile, Eigenschaften) von denen die
     * berechneten Kosten abhaengig sind.
     *
     * @return die Regelelemente von denen die Kosten abhaengen.
     */
    public Collection< Object > getAbhaengigkeiten();

    /**
     * Liefert die GP-Kosten fuer den Vor-/Nachteil. In vielen Faellen lassen
     * sich die Kosten nur ueber den Helden und den Wert genau bestimmten.
     * Fuer diese Faelle
     * existiert die Methode {@link #getGpKosten(Held, VorNachteilWert)}.
     * Lassen sich die Kosten ueber diese Methode hier nicht genau bestimmen, so
     * wird ein beliebiger Wert geliefert.
     *
     * @see #getGpKosten(VorNachteilWert)
     * @see #getGpKosten(Held,VorNachteilWert)
     * @return die GP-Kosten des Vor-/Nachteils.
     */
    public int getGpKosten();

    /**
     * Liefert die GP-Kosten fuer den Vor-/Nachteil. In einigen Faellen lassen
     * sich die Kosten nur ueber den Helden genau bestimmten. Fuer diese Faelle
     * existiert die Methode {@link #getGpKosten(Held, VorNachteilWert)}.
     * Lassen sich die Kosten ueber diese Methode hier nicht genau bestimmen, so
     * wird ein beliebiger Wert geliefert.
     *
     * @see #getGpKosten(Held, VorNachteilWert)
     * @param wert
     *            dessen Kosten ermittelt werden sollen.
     *
     * @return die GP-Kosten des Vor-/Nachteils.
     */
    public int getGpKosten( final VorNachteilWert wert );

    /**
     * Liefert die GP-Kosten die ein Held fuer einen Wert zahlen muss. Ist ein
     * Wert, z.B. durch Rasse, Kultur oder Profession vorgegeben, so sind seine
     * Kosten automatisch 0.
     *
     * @param held
     *            der die Kosten zahlen muss.
     * @param wert
     *            fuer den die Kosten gesucht sind.
     * @return die Kosten die {@code held} fuer {@code wert} zahlen muss.
     */
    public int getGpKosten( final Held held, final VorNachteilWert wert );

    /**
     * @param lernmethode
     *            die zum Senken der Schlechten Eigenschaft verwendet werden
     *            soll.
     * @return die AP die bei einer Schlechten Eigenschaft bezahlt werden
     *         muessen um den Wert um einen Punkt zu senken.
     */
    public int getSenkungskosten( final Lernmethode lernmethode );

    /**
     * Gibt bei einem Nachteil mit Stufen an ob er mit AP gesenkt werden kann.
     *
     * @return {@code true}, wenn der Nachteil gesenkt werden kann, sonst
     *         {@code false}.
     */
    public boolean istSenkbar();
}
