/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.stufen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;

/**
 * Fuer Vor-/Nachteile mit fester Mindest- und Hoechstufe.
 * 
 * @author Bernhard Jung
 */
class StufenStandard implements Stufen {

    /** Der kleinste noch erlaubte Wert. */
    public final int mindeststufe;
    
    /** Der groesset noch erlaubte Wert. */
    public final int hoechststufe;
    
    /** Die Schrittweite fuer die Stufe. */
    public final int schrittweite;
    
    /**
     * @param min der kleinste noch erlaubte Wert.
     * @param max der groesste noch erlaubte Wert.
     * @param schritt Die Schrittweite.
     */
    StufenStandard( final int min, final int max, final int schritt ) {
        
        mindeststufe = min;
        
        hoechststufe = max;
        
        schrittweite = schritt;
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }
    
    /** {@inheritDoc} */
    public final int getHoechststufe(Held held) {
        return hoechststufe;
    }

    /** {@inheritDoc} */
    public final int getMindeststufe() {
        return mindeststufe;
    }

    /** {@inheritDoc} */
    public final int getMindeststufe( final Held held ) {
        return mindeststufe;
    }

    /** {@inheritDoc} */
    public int getSchrittweite() {
        return schrittweite;
    }

    /** @return {@code true} */
    public boolean hatStufen() {
        return true;
    }

}
