/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.stufen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Bestimmt die Grenzwerte anhand des SO eines Helden.
 *
 * @author Bernhard Jung
 *
 */
class StufenVerbindungen implements Stufen {

    /** @return Soziale Anpassungsfaehigkeit. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList(
                (Object) VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT );
    }

    /**
     * Die Hoechststufe betraegt SO des Helden +5.
     * Mit Soziale Anpassungsfaehigkeit +7.
     * Der Wert ist nie groesser als 15.
     */
    public int getHoechststufe( final Held held ) {

        final int so = held.getStufe( Eigenschaft.SO );

        if ( held.hat( VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT ) ) {

            return Math.min( so + 7, 15 );

        } else {

            return Math.min( so + 5, 15 );

        }
    }

    /** @return 1 */
    public int getMindeststufe() {
        return 1;
    }

    /**
     * Die Mindeststufe betraegt SO des Helden -5.
     * Mit Soziale Anpassungsfaehigkeit SO - 7.
     * Sie ist nie kleiner als 1.
     */
    public int getMindeststufe(final Held held) {

        final int so = held.getStufe( Eigenschaft.SO );

        if ( held.hat( VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT ) ) {

            return Math.max( so - 7, 1 );

        } else {

            return Math.max( so - 5, 1 );

        }
    }

    /** @return 1 */
    public int getSchrittweite() {
        return 1;
    }

    /** @return {@code true} */
    public boolean hatStufen() {
        return true;
    }

}
