/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Eine Begabung fuer Schriften gibt es laut Regeln nicht.
 * 
 * @author Bernhard Jung
 *
 */
class VoraussetzungNichtBegabungSchrift extends Voraussetzung {

    private static final String KEINE_BEGABUNG_SCHRIFTEN =
        I18n.i18n( "VoraussetzungNichtBegabungSchriften" );
    
    /**
     * @return eine leere Liste.
     */
    @Override
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /**
     * Prueft ob Begabung fuer Schrift gewaehlt wurde und gibt ggf eine 
     * passende Meldung aus.
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( wert.getVariante() == Talentgruppe.SCHRIFTEN ) {
            
            beschreibung.einfuegen( KEINE_BEGABUNG_SCHRIFTEN );
        }
        
        return beschreibung;
    }

}
