/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Eine Voraussetzung fuer den Vorteil Viertelzauberer, der nicht von
 * Kaempferprofessionen waehlbar ist.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungNichtKaempferprofession extends Voraussetzung {

    /** Sowas wie "nicht waehlbar fuer Profession %1$s" */
    private final String NICHT_WAEHLBAR =
        I18n.i18n( "VoraussetzungNichtKaempferprofession" );

    /** @return eine leere Collection */
    @Override
    @SuppressWarnings( "unchecked" )
    public Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final Profession profession = held.getProfession();

        if ( profession.istKaempferprofession() ) {

            beschreibung.einfuegen( String.format( NICHT_WAEHLBAR,
                                                   profession.getName() ) );
        }

        return beschreibung;
    }

}
