/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

/**
 * Voraussetzung fuer Zauber die zu Beginn nur von einer bestimmten Profession
 * gewaehlt werden duerfen.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungBeginnProfession implements Voraussetzung {

    /** "zu Beginn nur %1$s" */
    private static final String ZU_BEGINN_NUR =
        I18n.i18n( "VoraussetzungBeginnProfession" );

    /** Die vorausgesetzte Profession. */
    private final Profession profession;

    /**
     * @param dieId der geforderten Profession.
     */
    VoraussetzungBeginnProfession( final String dieId ) {

        profession = Datenspeicher.getInstanz().getProfession( dieId );
    }

    /** @return eine leere Collection */
    @SuppressWarnings( "unchecked" )
    public final Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( final Held held, final ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.wirdErschaffen()
          && ! held.getProfession().equals( profession ) ) {

            beschreibung.einfuegen(
                    String.format( ZU_BEGINN_NUR, profession.getName() ) );
        }

        return beschreibung;
    }

}
